from typing import Callable

# 微信小程序配置
__WEIXIN_MP_CONFIG_CACHE__ = dict()

from afeng_tools.weixin_min_program.model.weixin_mp_enums import WeixinMpConfigItem, WeixinMpConfigKeyEnum


def init_mp_config(mp_app_id: str, mp_config_item: WeixinMpConfigItem):
    """初始化微信小程序配置"""
    _set_config(mp_app_id, WeixinMpConfigKeyEnum.mp_app_id, mp_config_item.mp_app_id)
    _set_config(mp_app_id, WeixinMpConfigKeyEnum.mp_app_secret, mp_config_item.mp_app_secret)
    _set_config(mp_app_id, WeixinMpConfigKeyEnum.mp_login_callback, mp_config_item.mp_login_callback)
    _set_config(mp_app_id, WeixinMpConfigKeyEnum.mp_get_login_info_callback, mp_config_item.mp_get_login_info_callback)
    _set_config(mp_app_id, WeixinMpConfigKeyEnum.mp_save_userinfo_callback, mp_config_item.mp_save_userinfo_callback)


def _set_config(mp_app_id: str, config_key: WeixinMpConfigKeyEnum, config_value: str | Callable):
    """设置配置"""
    if not __WEIXIN_MP_CONFIG_CACHE__.get(mp_app_id):
        __WEIXIN_MP_CONFIG_CACHE__[mp_app_id] = dict()
    __WEIXIN_MP_CONFIG_CACHE__[mp_app_id][config_key] = config_value


def get_config(mp_app_id: str, config_key: WeixinMpConfigKeyEnum) -> str | Callable:
    """获取配置"""
    if __WEIXIN_MP_CONFIG_CACHE__.get(mp_app_id):
        return __WEIXIN_MP_CONFIG_CACHE__.get(mp_app_id).get(config_key)



import os


def run(cmd, is_print: bool = True) -> list[str]:
    """
    运行命令
    :param cmd: 要执行的命令
    :param is_print: 是否在控制台打印信息
    :return: 命令输出信息
    """
    output = os.popen(cmd)
    line_list = output.buffer.read().decode('utf-8').split('\n')
    if is_print:
        for tmp_line in line_list:
            print(tmp_line)
    return line_list



"""
jinja2模板装饰器
"""
from functools import wraps

from afeng_tools.decorator_tool import decorator_tools


def jinja2_filter(name: str):
    """jinja2过滤器装饰器(name为jinja2模板中使用到的过滤器名)"""
    def func_wrap(func):
        decorator_tools.append_decorator_info(func, 'jinja2_filter')
        decorator_tools.append_func_info(func, '__filter_name__', name)

        @wraps(func)
        def inner_wrap(*args, **kwargs):
            return decorator_tools.run_func(func, *args, **kwargs)
        return inner_wrap
    return func_wrap



import math
import time
from typing import List

from appium import webdriver
from appium.webdriver import WebElement
from selenium.webdriver.support import expected_conditions
from selenium.webdriver.support.wait import WebDriverWait

from afeng_tools.appium_tool import mobile_adb_tools
from afeng_tools.appium_tool.appium_options_tools import huawei_options


class AppiumClientTool:
    def __init__(self, app_package: str, app_activity: str, appium_server_url: str = 'http://127.0.0.1:4723/wd/hub'):
        self.driver = webdriver.Remote(appium_server_url,
                                       options=huawei_options(app_package=app_package, app_activity=app_activity))

    def run_swipe(self, start_x, start_y, end_x, end_y) -> 'AppiumClientTool':
        """
        滑动屏幕
        :param start_x:
        :param start_y:
        :param end_x:
        :param end_y:
        :return: 当前类，用于链式操作
        """
        mobile_adb_tools.swipe(start_x=start_x, start_y=start_y, end_x=end_x, end_y=end_y)
        return self

    def get_viewport_rect(self):
        """
        获取当前页面的rect
        :return: 如：{'left': 0, 'top': 102, 'width': 1080, 'height': 2106}
        """
        return self.driver.capabilities['viewportRect']

    def get_viewport_size(self):
        viewport_rect = self.get_viewport_rect()
        return viewport_rect['width'], viewport_rect['height']

    def run_swipe_up(self, up_value: int = 300) -> 'AppiumClientTool':
        viewport_rect = self.get_viewport_rect()
        left, top = viewport_rect['left'], viewport_rect['top']
        width, height = viewport_rect['width'], viewport_rect['height']
        x = left + math.floor(width / 2)
        y = top + math.floor(height / 2)
        mobile_adb_tools.swipe(start_x=x, start_y=y, end_x=x, end_y=y - up_value if y - up_value > top else top)
        return self

    def run_swipe_down(self, down_value: int = 300) -> 'AppiumClientTool':
        viewport_rect = self.get_viewport_rect()
        left, top = viewport_rect['left'], viewport_rect['top']
        width, height = viewport_rect['width'], viewport_rect['height']
        x = left + math.floor(width / 2)
        y = top + math.floor(height / 2)
        mobile_adb_tools.swipe(start_x=x, start_y=y, end_x=x, end_y=y + down_value if y + down_value > top else top)
        return self

    def run_click_point(self, x: int, y: int) -> 'AppiumClientTool':
        """
        点击屏幕某个坐标点
        :param x:  坐标x
        :param y: 坐标y
        :return: 当前类，用于链式操作
        """
        mobile_adb_tools.click(x=x, y=y)
        return self

    def run_click(self, by: str, value: str) -> WebElement:
        """
        点击按钮
        :param by: 元素查询条件
        :param value: 查询条件值
        :return: WebElement 点击的元素
        """
        tmp_el = self.driver.find_element(by, value)
        tmp_el.click()
        return tmp_el

    def run_click_elements_one(self, by: str, value: str, index: int) -> WebElement:
        """
        点击按钮集合的第index个
        :param index:
        :param by: 元素查询条件
        :param value: 查询条件值
        @index: 点击元素的索引号
        :return: WebElement 点击的元素
        """
        tmp_el = self.driver.find_elements(by, value)[index]
        tmp_el.click()
        return tmp_el

    def run_wait(self, wait_time: float) -> 'AppiumClientTool':
        """
        等待一些时间
        :param wait_time: 等待的时间：（单位：秒）
        :return: 当前类，用于链式操作
        """
        self.driver.implicitly_wait(wait_time)
        return self

    def run_sleep(self, seconds: float) -> 'AppiumClientTool':
        """
        睡眠一些时间
        :param seconds: 睡眠秒数
        :return: 当前类，用于链式操作
        """
        time.sleep(seconds)
        return self

    def run_wait_element(self, by: str, value: str, timeout: float) -> WebElement:
        """
        等待元素出现
        :param by: 元素查询条件
        :param value: 查询条件值
        :param timeout: 超时时间
        :return: 等待到的元素
        """
        wait = WebDriverWait(self.driver, timeout)
        return wait.until(expected_conditions.presence_of_element_located((by, value)))

    def run_find_element(self, by: str, value: str) -> WebElement:
        """
        查找单个元素
        :param by: 元素查询条件
        :param value: 查询条件值
        :return: WebElement
        """
        return self.driver.find_element(by, value)

    def run_find_elements(self, by: str, value: str) -> List[WebElement]:
        """
        查找多个个元素
        :param by: 元素查询条件
        :param value: 查询条件值
        :return: list[WebElement]
        """
        return self.driver.find_elements(by, value)

    def run_wait_activity(self, activity_name, timeout: int = 3) -> bool:
        """
        等待 activity 加载： 在给定的超时时间内，通过不断轮询Activity的方式，等待指定的Android Activity出现
        :param activity_name: activity名称
        :param timeout: 最大等待时间（单位：秒）
        :return: 是否等待到 activity， False是没有等到，True等到了
        """
        return self.driver.wait_activity(activity_name, timeout)



from typing import Optional
from pydantic import BaseModel


class WxMpLoginInfo(BaseModel):
    """微信小程序登录信息"""
    # 小程序 appId
    mp_app_id: Optional[str] = None
    # 会话密钥
    session_key: Optional[str] = None
    # 用户在开放平台的唯一标识符，若当前小程序已绑定到微信开放平台账号下会返回，详见 UnionID 机制说明。
    union_id: Optional[str] = None
    # 用户唯一标识
    open_id: Optional[str] = None


class WxMpUserInfo(BaseModel):
    """微信小程序用户信息"""
    # 认证token
    token: Optional[str] = None
    nick_name: Optional[str] = None
    avatar_url: Optional[str] = None
    city: Optional[str] = None
    country: Optional[str] = None
    province: Optional[str] = None
    gender: Optional[int] = None
    language: Optional[str] = None



"""
百度网盘文件路径工具
"""
from typing import List

import requests

import openapi_client
from openapi_client.api.fileinfo_api import FileinfoApi
from openapi_client.api.multimediafile_api import MultimediafileApi
from afeng_tools.baidu_pan_tool.core.baidu_pan_decorator_tools import auto_fileinfo_api, auto_media_file_api
from afeng_tools.baidu_pan_tool.core.baidu_pan_models import FileInfo, DocFileInfo, ImageInfo, SearchFileInfo, SearchResultInfo, \
    MultimediaFileInfo, VideoInfo, BtInfo, CategoryListResult, CategoryListItem
from afeng_tools.http_tool import http_params_tools
from afeng_tools.log_tool.loguru_tools import get_logger, log_error

logger = get_logger()


@auto_media_file_api
def list_all_file(access_token: str, dir_path: str, recursion: int = 0,
                  order: str = 'name', desc: int = 0, web: int = 0, only_file: bool = False,
                  api_instance: MultimediafileApi = None) -> List[FileInfo]:
    """
    递归获取文件列表: 本接口可以递归获取指定目录下的所有文件列表（包括目录）。
    :param access_token: Access Token
    :param dir_path: 目录名称绝对路径，必须/开头；
    :param recursion: 是否递归，0为否，1为是，默认为0, 当目录下存在文件夹，并想获取到文件夹下的子文件时，可以设置 recursion 参数为1, 即可获取到更深目录层级的文件。
    :param order: 排序字段
                    time(修改时间)
                    name(文件名，注意，此处排序是按字符串排序的)
                    size(大小，目录无大小)，默认为文件类型
    :param desc: 0为升序，1为降序，默认为0
    :param web: 默认为0， 为1时返回缩略图地址
    :param only_file: 是否只有文件
    :param api_instance: 自动注入MultimediafileApi
    :return: List[FileInfo]
    """
    tmp_file_list = []
    _result = list_all_by_page(access_token, dir_path=dir_path, recursion=recursion, order=order, desc=desc,
                               web=web, api_instance=api_instance)
    if _result and _result.file_list:
        tmp_file_list.extend(_result.file_list)
        while (_result.errno == 0 or _result.errno is None) and _result.has_more:
            _result = list_all_by_page(access_token, dir_path=dir_path, recursion=recursion, order=order, desc=desc,
                                       start=_result.cursor, web=web, api_instance=api_instance)
            if _result and _result.file_list:
                tmp_file_list.extend(_result.file_list)
        if only_file:
            tmp_file_list = filter(lambda x: x.isdir == 0, tmp_file_list)
    return tmp_file_list


@auto_media_file_api
def list_all_by_page(access_token: str, dir_path: str, recursion: int = 0,
                     order: str = 'name', desc: int = 0, start: int = 0, limit: int = 1000,
                     web: int = 0,
                     api_instance: MultimediafileApi = None) -> MultimediaFileInfo:
    """
    获取文件列表（前1000条）: 本接口可以递归获取指定目录下的文件列表。
    :param access_token: Access Token
    :param dir_path: 目录名称绝对路径，必须/开头；
    :param recursion: 是否递归，0为否，1为是，默认为0, 当目录下存在文件夹，并想获取到文件夹下的子文件时，可以设置 recursion 参数为1, 即可获取到更深目录层级的文件。
    :param order: 排序字段
                    time(修改时间)
                    name(文件名，注意，此处排序是按字符串排序的)
                    size(大小，目录无大小)，默认为文件类型
    :param desc: 0为升序，1为降序，默认为0
    :param start: 查询起点，默认为0，当返回has_more=1时，应使用返回的cursor作为下一次查询的起点
    :param limit: 查询数目，默认为1000； 如果设置start和limit参数，则建议最大设置为1000
    :param web: 默认为0， 为1时返回缩略图地址
    :param api_instance: 自动注入MultimediafileApi
    :return: MultimediaFileInfo
    """
    try:
        api_response = api_instance.xpanfilelistall(access_token, dir_path, recursion,
                                                    order=order, desc=desc, start=start, limit=limit,
                                                    web=str(web))
        if api_response['errno'] == 0:

            multi_info = MultimediaFileInfo(
                has_more=api_response['has_more'] == 1,
                cursor=api_response['cursor']
            )
            multi_info.file_list = [FileInfo(**tmp_data) for tmp_data in api_response['list']]
            return multi_info
        elif api_response['errno'] == 42213:
            log_error(logger, f"[BaiduPan]文件或目录[{dir_path}]无权访问搜索")
        elif api_response['errno'] == 31066:
            log_error(logger, f"[BaiduPan]文件或目录[{dir_path}]不存在，无法进行搜索")
        elif api_response['errno'] == 31034:
            log_error(logger, f"[{dir_path}]命中频控,listall接口的请求频率建议不超过每分钟8-10次")
        else:
            log_error(logger, f"[BaiduPan]搜索目录[{dir_path}]下文件列表失败，api_response: {api_response}")
    except openapi_client.ApiException as e:
        log_error(logger, "[BaiduPan]Exception when calling MultimediafileApi->xpanfilelistall", e)


@auto_fileinfo_api
def list_file_by_page(access_token: str, dir_path: str = '/', order: str = 'name', desc: int = 0,
                      start: int = 0, limit: int = 1000, web: int = 0, folder: int = 0, show_empty: int = 1,
                      api_instance: FileinfoApi = None) -> list[FileInfo]:
    """
    获取文件列表（前1000条）: 本接口用于获取用户网盘中指定目录下的文件列表。返回的文件列表支持排序、分页等操作。
    :param access_token: Access Token
    :param dir_path: 需要list的目录，以/开头的绝对路径, 默认为/
    :param order: 排序字段：默认为name；
                time表示先按文件类型排序，后按修改时间排序；
                name表示先按文件类型排序，后按文件名称排序；(注意，此处排序是按字符串排序的)
                size表示先按文件类型排序，后按文件大小排序。
    :param desc: 默认为升序，设置为1实现降序, 排序的对象是当前目录下所有文件，不是当前分页下的文件
    :param start: 起始位置，从0开始
    :param limit: 查询数目，默认为1000，建议最大不超过1000
    :param web: 值为1时，返回dir_empty属性和缩略图数据
    :param folder: 是否只返回文件夹，0 返回所有，1 只返回文件夹，且属性只返回path字段
    :param show_empty: 是否返回dir_empty属性，0 不返回，1 返回
    :param api_instance: 自动注入的 FileinfoApi
    :return:
    """
    try:
        api_response = api_instance.xpanfilelist(access_token, dir=dir_path,
                                                 start=str(start), limit=limit,
                                                 order=order, desc=desc, web=str(web),
                                                 folder=str(folder), showempty=show_empty)
        if api_response['errno'] == 0:
            return [FileInfo(**tmp_data) for tmp_data in api_response['list']]
        elif api_response['errno'] == -7:
            log_error(logger, f"文件或目录[{dir_path}]无权访问，响应信息：{api_response}")
        elif api_response['errno'] == -9:
            log_error(logger, f"文件或目录[{dir_path}]不存在，响应信息：{api_response}")
        else:
            log_error(logger, f"获取目录[{dir_path}]下文件列表失败，响应信息：{api_response}")
    except openapi_client.ApiException as e:
        log_error(logger, "[BaiduPan]Exception when calling FileinfoApi->xpanfilelist", e)


@auto_fileinfo_api
def list_image(access_token: str, dir_path: str = '/',
               page_num: int = None, page_size: int = 1000,
               order: str = 'name', desc: int = 1,
               recursion: int = 0, web: int = 0,
               api_instance: FileinfoApi = None) -> List[ImageInfo]:
    """
     获取图片列表（前1000条）: 获取用户指定目录下的图片列表
     :param access_token: Access Token
     :param dir_path: 目录名称，以/开头的绝对路径, 默认为/ , 路径包含中文时需要UrlEncode编码
     :param page_num: 页码，从1开始， 如果不指定页码，则为不分页模式，返回所有的结果。如果指定page参数，则按修改时间倒序排列
     :param page_size: 一页返回的文档数， 默认值为1000，建议最大值不超过1000
     :param order:  排序字段：默认为name
                        time按修改时间排序，
                        name按文件名称排序，
                        size按文件大小排序，
     :param desc: 0为升序，1为降序，默认为1
     :param recursion:  是否需要递归，0为不需要，1为需要，默认为0, 递归是指：当目录下有文件夹，使用此参数，可以获取到文件夹下面的文档
     :param web: 为1时返回文档预览地址lodocpreview
     :param api_instance: 自动注入的 FileinfoApi
     :return: List[ImageInfo]
     """
    try:
        api_response = api_instance.xpanfileimagelist(access_token, parent_path=dir_path,
                                                      recursion=str(recursion),
                                                      page=page_num, num=page_size,
                                                      order=order, desc=desc, web=str(web))
        if api_response['errno'] == 0:
            return [ImageInfo(**tmp_data) for tmp_data in api_response['info']]
        else:
            log_error(logger, f"[BaiduPan]获取目录[{dir_path}]下图片列表失败:{api_response}")
    except openapi_client.ApiException as e:
        log_error(logger, "[BaiduPan]Exception when calling FileinfoApi->xpanfileimagelist", e)


@auto_fileinfo_api
def list_doc(access_token: str, dir_path: str = '/',
             page_num: int = None, page_size: int = 1000,
             order: str = 'name', desc: int = 1,
             recursion: int = 0, web: int = 0,
             api_instance: FileinfoApi = None) -> List[DocFileInfo]:
    """
    获取文档列表（前1000条）：获取用户指定目录下的文档列表。
    :param access_token: Access Token
    :param dir_path: 目录名称，以/开头的绝对路径, 默认为/
    :param page_num: 页码，从1开始， 如果不指定页码，则为不分页模式，返回所有的结果。如果指定page参数，则按修改时间倒序排列
    :param page_size: 一页返回的文档数， 默认值为1000，建议最大值不超过1000
    :param order:  排序字段： 默认为name
                        time按修改时间排序
                        name按文件名称排序
                        size按文件大小排序
    :param desc: 0为升序，1为降序，默认为1
    :param recursion:  是否需要递归，0为不需要，1为需要，默认为0, 递归是指：当目录下有文件夹，使用此参数，可以获取到文件夹下面的文档
    :param web: 为1时返回文档预览地址lodocpreview
    :param api_instance: 自动注入的 FileinfoApi
    :return: List[DocFileInfo]
    """
    try:
        api_response = api_instance.xpanfiledoclist(access_token, parent_path=dir_path, order=order, desc=desc,
                                                    recursion=str(recursion), page=page_num, num=page_size,
                                                    web=str(web))
        if api_response['errno'] == 0:
            return [DocFileInfo(**tmp_data) for tmp_data in api_response['info']]
        else:
            log_error(logger, f"[BaiduPan][{dir_path}]下文档列表失败: {api_response}")
    except openapi_client.ApiException as e:
        log_error(logger, "[BaiduPan]Exception when calling FileinfoApi->xpanfiledoclist", e)


def list_video(access_token: str, dir_path: str = '/',
               page_num: int = None, page_size: int = 1000,
               order: str = 'name', desc: int = 1,
               recursion: int = 0, web: int = 0) -> list[VideoInfo]:
    """
    获取视频列表: 本接口用于获取用户指定目录下的视频列表.
    :param access_token: Access Token
    :param dir_path: 目录名称，以/开头的绝对路径, 默认为/
    :param page_num: 页码，从1开始， 如果不指定页码，则为不分页模式，返回所有的结果。如果指定page参数，则按修改时间倒序排列
    :param page_size: 一页返回的文件数， 默认值为1000, 最大值建议不超过1000
    :param order: 排序字段， 默认为name
                    time按修改时间排序
                    name按文件名称排序(注意，此处排序是按字符串排序的）
                    size按文件大小排序
    :param desc: 0为升序，1为降序，默认为1
    :param recursion: 是否需要递归，0为不需要，1为需要，默认为0
                        递归是指：当目录下有文件夹，使用此参数，可以获取到文件夹下面的视频
    :param web: 为1时返回视频预览缩略图
    :return: list[VideoInfo]
    """
    params = {
        'method': 'videolist',
        'access_token': access_token,
        'parent_path': http_params_tools.url_encode(dir_path),
        'page': page_num,
        'num': page_size,
        'order': order,
        'desc': desc,
        'recursion': recursion,
        'web': web
    }
    headers = {
        'User-Agent': 'pan.baidu.com'
    }
    try:
        response = requests.get('http://pan.baidu.com/rest/2.0/xpan/file', params=params, headers=headers)
        response.encoding = 'utf8'
        api_response = response.json()
        if api_response['errno'] == 0:
            return [VideoInfo(**tmp_data) for tmp_data in api_response['info']]
        else:
            log_error(logger, f"[BaiduPan][{dir_path}]下视频列表失败: {api_response}")
    except openapi_client.ApiException as e:
        log_error(logger, "[BaiduPan]Exception when calling FileinfoApi->xpanfilevideolist", e)


def list_bt(access_token: str, dir_path: str = '/',
            page_num: int = None, page_size: int = 1000,
            order: str = 'name', desc: int = 1,
            recursion: int = 0) -> list[BtInfo]:
    """
    获取bt列表: 本接口用于获取用户指定路径下的bt文件列表。
    :param access_token: Access Token
    :param dir_path: 目录名称，以/开头的绝对路径, 默认为/
    :param page_num: 页码，从1开始， 如果不指定页码，则为不分页模式，返回所有的结果。如果指定page参数，则按修改时间倒序排列
    :param page_size: 一页返回的文件数， 默认值为1000, 最大值建议不超过1000
    :param order: 排序字段， 默认为name
                    time按修改时间排序
                    name按文件名称排序(注意，此处排序是按字符串排序的）
                    size按文件大小排序
    :param desc: 0为升序，1为降序，默认为1
    :param recursion: 是否需要递归，0为不需要，1为需要，默认为0
                        递归是指：当目录下有文件夹，使用此参数，可以获取到文件夹下面的bt文件
    :return: list[BtInfo]
    """
    params = {
        'method': 'btlist',
        'access_token': access_token,
        'parent_path': http_params_tools.url_encode(dir_path),
        'page': page_num,
        'num': page_size,
        'order': order,
        'desc': desc,
        'recursion': recursion,
    }
    headers = {
        'User-Agent': 'pan.baidu.com',
        'Cookie': 'PANWEB=1; BAIDUID=AC26BE01592777C2F2253ECBC0E5780B:FG=1'
    }
    try:
        response = requests.get('http://pan.baidu.com/rest/2.0/xpan/file', params=params, headers=headers)
        response.encoding = 'utf8'
        api_response = response.json()
        if api_response['errno'] == 0:
            return [BtInfo(**tmp_data) for tmp_data in api_response['info']]
        else:
            log_error(logger, f"[BaiduPan][{dir_path}]下bt列表失败: {api_response}")
    except openapi_client.ApiException as e:
        log_error(logger, "[BaiduPan]Exception when calling FileinfoApi->xpanfilebtlist", e)


@auto_fileinfo_api
def search(access_token: str, key_word: str, dir_path: str = '/', category: int = None,
           page_num: str = None, page_size: int = 500,
           recursion: str = '0', web: str = '0', device_id: str = None,
           api_instance: FileinfoApi = None) -> SearchResultInfo:
    """
    搜索文件: 本接口用于获取用户指定目录下，包含指定关键字的文件列表。
    :param access_token: Access T
    :param key_word: 搜索关键字，最大30字符（UTF8格式）
    :param dir_path: 搜索目录，默认根目录
    :param category: 文件类型，1 视频、2 音频、3 图片、4 文档、5 应用、6 其他、7 种子
    :param page_num: 页数，从1开始，缺省则返回所有条目
    :param page_size: 默认为500，不能修改
    :param recursion: 是否递归，带这个参数就会递归，否则不递归
    :param web: 是否展示缩略图信息，带这个参数会返回缩略图信息，否则不展示缩略图信息
    :param device_id: 设备ID，设备注册接口下发，硬件设备必传
    :param api_instance: 自动注入的 FileinfoApi
    :return: SearchResultInfo
    """
    try:
        api_response = api_instance.xpanfilesearch(access_token, key=key_word, dir=dir_path, category=category,
                                                   num=page_num, page=page_size,
                                                   recursion=recursion, web=web,
                                                   device_id=device_id)
        if api_response['errno'] == 0:
            result = SearchResultInfo(has_more=api_response['has_more'] == 1,
                                      content_list=api_response['contentlist'])
            result.file_list = [SearchFileInfo(**tmp_data) for tmp_data in api_response['list']]
            return result
        elif api_response['errno'] == -7:
            log_error(logger, f"[BaiduPan]目录[{dir_path}]无权访问搜索")
        elif api_response['errno'] == -9:
            log_error(logger, f"[BaiduPan]目录[{dir_path}]不存在，无法进行搜索")
        else:
            log_error(logger, f"[BaiduPan]搜索目录[{dir_path}]下文件[{key_word}]失败: {api_response}")
    except openapi_client.ApiException as e:
        log_error(logger, "[BaiduPan]Exception when calling FileinfoApi->xpanfilesearch", e)


def list_category_file(access_token: str, category: int, dir_path: str = '/',
                       show_dir: int = 0, recursion: int = 0, ext: str = None,
                       start: int = 0, limit: int = 1000,
                       order: str = 'name', desc: int = 1, device_id: str = None) -> CategoryListResult:
    """
    获取分类文件列表: 本接口用于获取用户目录下指定类型的文件列表。
    :param access_token: Access Token
    :param category: 文件类型，1 视频、2 音频、3 图片、4 文档、5 应用、6 其他、7 种子, 多个category使用英文逗号分隔，示例：3,4
    :param dir_path: 目录名称，以/开头的绝对路径, 默认为/
    :param show_dir: 是否展示文件夹，0:否(默认) 1:是
    :param recursion: 是否需要递归，0为不需要，1为需要，默认为0 （注意recursion=1时不支持show_dir=1）
    :param ext: 需要的文件格式，多个格式以英文逗号分隔，示例: txt,epub，默认为category下所有格式
    :param start: 查询起点，默认为0
    :param limit: 查询数目，最大1000，默认1000
    :param order: 排序字段， 默认为name
                    time按修改时间排序
                    name按文件名称排序(注意，此处排序是按字符串排序的）
                    size按文件大小排序
    :param desc: 0为升序，1为降序，默认为1
    :param device_id: 设备ID，硬件设备必传
    :return: CategoryListResult
    """
    params = {
        'method': 'categorylist',
        'access_token': access_token,
        'category': category,
        'show_dir': show_dir,
        'parent_path': http_params_tools.url_encode(dir_path),
        'recursion': recursion,
        'ext': ext,
        'start': start,
        'limit': limit,
        'order': order,
        'desc': desc,
        'device_id': device_id
    }
    headers = {
        'User-Agent': 'pan.baidu.com'
    }
    try:
        response = requests.get('http://pan.baidu.com/rest/2.0/xpan/multimedia', params=params, headers=headers)
        response.encoding = 'utf8'
        api_response = response.json()
        if api_response['errno'] == 0:
            result = CategoryListResult(has_more=api_response['has_more'] == 1,
                                        cursor=api_response['cursor'])
            result.file_list = [CategoryListItem(**tmp_data) for tmp_data in api_response['list']]
            return result
        else:
            log_error(logger, f"[BaiduPan]获取[{dir_path}]下分类[{category}]文件列表失败: {api_response}")
    except openapi_client.ApiException as e:
        log_error(logger, "[BaiduPan]Exception when calling multimedia->categorylist", e)



from enum import Enum
from typing import Callable, Optional

from pydantic import BaseModel

from afeng_tools.weixin_min_program.model.weixin_mp_open_models import WxMpLoginInfo, WxMpUserInfo


class WeixinMpConfigItem(BaseModel):
    """微信小程序配置"""
    # 小程序 appId
    mp_app_id: Optional[str] = None
    # 小程序 appSecret
    mp_app_secret: Optional[str] = None
    # 小程序登录Callback
    mp_login_callback: Optional[Callable[[WxMpLoginInfo], str]] = None
    # 小程序获取登录信息Callback
    mp_get_login_info_callback: Optional[Callable[[str], WxMpLoginInfo]] = None
    #  小程序保存用户信息Callback
    mp_save_userinfo_callback: Optional[Callable[[WxMpUserInfo], dict]] = None


class WeixinMpConfigKeyEnum(Enum):
    """微信配置枚举"""
    # 小程序 appId
    mp_app_id = 'mp_app_id'
    # 小程序 appSecret
    mp_app_secret = 'mp_app_secret'
    # 小程序登录Callback
    mp_login_callback = 'mp_login_callback'
    # 小程序获取登录信息Callback
    mp_get_login_info_callback = 'mp_get_login_info_callback'
    # 小程序保存用户信息Callback
    mp_save_userinfo_callback = 'mp_save_userinfo_callback'




from afeng_tools.re_tool.re_tools import get_match_first_group


def parse_to_user(msg):
    return get_match_first_group(r'<ToUserName><!\[CDATA\[(.*?)\]\]></ToUserName>', msg)


def parse_from_user(msg):
    return get_match_first_group(r'<FromUserName><!\[CDATA\[(.*?)\]\]></FromUserName>', msg)


def parse_create_time(msg):
    return get_match_first_group(r'<CreateTime>(.*?)</CreateTime>', msg)


def parse_msg_type(msg):
    return get_match_first_group(r'<MsgType><!\[CDATA\[(.*?)\]\]></MsgType>', msg)


def parse_msg_content(msg):
    return get_match_first_group(r'<Content><!\[CDATA\[(.*?)\]\]></Content>', msg)


def parse_msg_id(msg):
    return get_match_first_group(r'<MsgId>(.*?)</MsgId>', msg)


def parse_msg_data_id(msg):
    return get_match_first_group(r'<MsgDataId>(.*?)</MsgDataId>', msg)


def parse_idx(msg):
    return get_match_first_group(r'<Idx>(.*?)</Idx>', msg)


def parse_pic_url(msg):
    return get_match_first_group(r'<PicUrl><!\[CDATA\[(.*?)\]\]></PicUrl>', msg)


def parse_media_id(msg):
    return get_match_first_group(r'<MediaId><!\[CDATA\[(.*?)\]\]></MediaId>', msg)


def parse_voice_format(msg):
    return get_match_first_group(r'<Format><!\[CDATA\[(.*?)\]\]></Format>', msg)


def parse_recognition(msg):
    return get_match_first_group(r'<Recognition><!\[CDATA\[(.*?)\]\]></Recognition>', msg)


def parse_thumb_media_id(msg):
    return get_match_first_group(r'<ThumbMediaId><!\[CDATA\[(.*?)\]\]></ThumbMediaId>', msg)


def parse_location_x(msg):
    return get_match_first_group(r'<Location_X>(.*?)</Location_X>', msg)


def parse_location_y(msg):
    return get_match_first_group(r'<Location_Y>(.*?)</Location_Y>', msg)


def parse_location_scale(msg):
    return get_match_first_group(r'<Scale>(.*?)</Scale>', msg)


def parse_location_label(msg):
    return get_match_first_group(r'<Label><!\[CDATA\[(.*?)\]\]></Label>', msg)


def parse_link_title(msg):
    return get_match_first_group(r'<Title><!\[CDATA\[(.*?)\]\]></Title>', msg)


def parse_link_description(msg):
    return get_match_first_group(r'<Description><!\[CDATA\[(.*?)\]\]></Description>', msg)


def parse_link_url(msg):
    return get_match_first_group(r'<Url><!\[CDATA\[(.*?)\]\]></Url>', msg)



from sqlalchemy import Column, Integer, String

from afeng_tools.sqlalchemy_tool.core.sqlalchemy_base_model import Model


class WeixinEventModel(Model):
    """微信事件基本信息表"""
    __abstract__ = True
    openid = Column(String(125), comment='OpenID', nullable=False)
    to_user = Column(String(250), comment='开发者微信号', nullable=False)
    from_user = Column(String(250), comment='发送方帐号（一个OpenID）', nullable=False)
    create_time = Column(Integer, comment='消息创建时间 （整型）', nullable=False)
    # 消息类型，event
    msg_type = Column(String(30), comment='消息类型，event', nullable=False)


class WeixinSubscribeEvent(WeixinEventModel):
    """订阅/取消订阅事件表"""
    __tablename__ = 'weixin_subscribe_event'
    event = Column(String(20), comment='事件类型，subscribe(订阅)、unsubscribe(取消订阅)', nullable=False)

    def __repr__(self):
        return self.event


class WeixinQrEvent(WeixinEventModel):
    """扫描带参数二维码事件"""
    __tablename__ = 'weixin_qr_event'
    event = Column(String(20), comment='事件类型，subscribe', nullable=False)
    event_key = Column(String(500), comment='事件 KEY 值，qrscene_为前缀，后面为二维码的参数值', nullable=False)
    ticket = Column(String(250), comment='二维码的ticket，可用来换取二维码图片', nullable=False)

    def __repr__(self):
        return self.event_key


class WeixinScanEvent(WeixinEventModel):
    """用户已关注时的事件推送"""
    __tablename__ = 'weixin_scan_event'
    event = Column(String(20), comment='事件类型，SCAN', nullable=False)
    event_key = Column(String(64), comment='事件 KEY 值，是一个32位无符号整数，即创建二维码时的二维码scene_id', nullable=False)
    ticket = Column(String(250), comment='二维码的ticket，可用来换取二维码图片', nullable=False)

    def __repr__(self):
        return self.event_key


class WeixinLocationEvent(WeixinEventModel):
    """上报地理位置事件"""
    __tablename__ = 'weixin_location_event'
    event = Column(String(20), comment='事件类型，LOCATION', nullable=False)
    location_x = Column(String(20), comment='地理位置纬度', nullable=False)
    location_y = Column(String(20), comment='地理位置经度', nullable=False)
    precision = Column(String(20), comment='地理位置精度', nullable=False)

    def __repr__(self):
        return self.location_x, self.location_y


class WeixinMenuEvent(WeixinEventModel):
    """点击自定义菜单事件"""
    __tablename__ = 'weixin_menu_event'
    event = Column(String(20), comment='事件类型，CLICK:点击菜单拉取消息, VIEW:点击菜单跳转链接时的事件推送', nullable=False)
    event_key = Column(String(255), comment='事件 KEY 值，与自定义菜单接口中KEY值对应 或 设置的跳转URL', nullable=False)

    def __repr__(self):
        return self.event_key





import logging
import os
from pathlib import Path

from afeng_tools.log_tool import logger_settings
from afeng_tools.log_tool.logger_enums import LoggerConfigKeyEnum


def get_logger(log_name, store: bool = True):
    """
    创建logger
    :param log_name: 日志名称
    :param log_file: 日志文件
    :param store: 存储日志至本地
    :return: Logger
    """
    log_file = logger_settings.get_config(LoggerConfigKeyEnum.info_file)
    logger = logging.getLogger(log_name)
    logger.setLevel(logging.INFO)
    stream = logging.StreamHandler()
    stream.setLevel(logging.INFO)
    formatter = logging.Formatter('[%(levelname)s %(asctime)s] %(message)s')
    stream.setFormatter(formatter)
    logger.addHandler(stream)
    if store is True:
        os.makedirs(Path(log_file).parent, exist_ok=True)
        logfile = logging.FileHandler(log_file, mode='a')
        logfile.setLevel(logging.INFO)
        logfile.setFormatter(formatter)
        logger.addHandler(stream)
    return logger



from playwright.sync_api import Page

from afeng_tools.playwright_tool.decorator.playwright_decorators import auto_input


@auto_input(headless=False)
def run_test(web_page: Page = None):
    web_page.goto('https://data.eastmoney.com/stockcomment/stock_code/688981.html')
    print(web_page.title())
    # 笔画详情
    for i, tmp_stroke_unit in enumerate(web_page.locator('.word-stroke-detail').all()):
        tmp_stroke_unit.locator('g').screenshot(path=f'output/{i + 1}.png')


@auto_input(headless=False)
def run_test(stock_code, web_page: Page = None):
    web_page.goto('https://data.eastmoney.com/stockcomment/stock_code/688981.html')
    print(web_page.title())
    # 笔画详情
    for i, tmp_stroke_unit in enumerate(web_page.locator('.word-stroke-detail').all()):
        tmp_stroke_unit.locator('g').screenshot(path=f'output/{i + 1}.png')


if __name__ == '__main__':
    run_test()



from afeng_tools.encryption_tool import hashlib_tools

from afeng_tools.web_tool import response_tools
from afeng_tools.web_tool.core.web_common_models import ResponseModel
from afeng_tools.weixin_min_program import weixin_mp_tools, weixin_mp_settings
from afeng_tools.weixin_min_program.core.weixin_mp_api_service import request_jscode2session
from afeng_tools.weixin_min_program.core.weixin_mp_models import WxUserProfile
from afeng_tools.weixin_min_program.model.weixin_mp_enums import WeixinMpConfigKeyEnum
from afeng_tools.weixin_min_program.model.weixin_mp_open_models import WxMpLoginInfo, WxMpUserInfo

# 缓存
__LOCAL_CACHE__ = dict()


def login_code2Session(mp_app_id: str, js_code: str) -> ResponseModel:
    result = request_jscode2session(mp_app_id, js_code)
    if result.errcode == 40029:
        return response_tools.create_json_response_data(error_no=429, message='js_code无效')
    elif result.errcode == 40226:
        return response_tools.create_json_response_data(error_no=4226, message='高风险等级用户, 无法登录小程序')
    elif result.errcode is None or result.errcode == 0:
        login_info = WxMpLoginInfo(
            mp_app_id=mp_app_id,
            union_id=result.unionid,
            open_id=result.openid,
            session_key=result.session_key,
        )
        login_callback = weixin_mp_settings.get_config(mp_app_id, WeixinMpConfigKeyEnum.mp_login_callback)
        if login_callback:
            token = login_callback(login_info)
        else:
            token = hashlib_tools.calc_md5(login_info.mp_app_id + login_info.open_id + login_info.union_id)
        __LOCAL_CACHE__[token] = login_info
        return response_tools.create_json_response_data(data=token)
    else:
        return login_code2Session(mp_app_id, js_code)


def decrypt_userinfo(mp_app_id: str, token: str, user_profile: WxUserProfile) -> ResponseModel:
    get_login_info_callback = weixin_mp_settings.get_config(mp_app_id, WeixinMpConfigKeyEnum.mp_get_login_info_callback)
    if get_login_info_callback:
        login_info = get_login_info_callback(token)
    else:
        login_info = __LOCAL_CACHE__.get(token)
    session_key = login_info.session_key
    if user_profile.signature == weixin_mp_tools.calc_signature(session_key=session_key, raw_data=user_profile.rawData):
        user_data = weixin_mp_tools.decrypt_data(app_id=mp_app_id,
                                                 session_key=session_key,
                                                 encrypted_data=user_profile.encryptedData, iv_value=user_profile.iv)
        # 保存用户信息
        save_userinfo_callback = weixin_mp_settings.get_config(mp_app_id,
                                                               WeixinMpConfigKeyEnum.mp_save_userinfo_callback)
        if save_userinfo_callback:
            user_data = save_userinfo_callback(WxMpUserInfo(
                token=token,
                nick_name=user_data.nickName,
                avatar_url=user_data.avatarUrl,
                city=user_data.city,
                country=user_data.country,
                province=user_data.province,
                gender=user_data.gender,
                language=user_data.language,
            ))
        return response_tools.create_json_response_data(data=user_data)
    return response_tools.create_json_response_data(error_no=401, message='认证失败')



"""
二维码工具: pip install qrcode -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
"""
import base64
import io

import qrcode
import qrcode.image.svg
from qrcode.compat.pil import Image


def get_qrcode_svg_factory(method: str = None):
    """获取svg的factory"""
    if method == 'basic':
        # Simple factory, just a set of rects.
        factory = qrcode.image.svg.SvgImage
    elif method == 'fragment':
        # Fragment factory (also just a set of rects)
        factory = qrcode.image.svg.SvgFragmentImage
    else:
        # Combined path factory, fixes white space that may occur when zooming
        factory = qrcode.image.svg.SvgPathImage
    return factory


def make_qrcode_svg(data_str: str, save_svg: str, method: str = None) -> Image:
    """创建二维码svg"""
    pil_img = qrcode.make(data_str, image_factory=get_qrcode_svg_factory(method=method))
    pil_img.save(save_svg)
    return pil_img


def make_qrcode(data_str: str, save_image: str, option: dict = None) -> Image:
    """
    创建二维码
    :param data_str: 数据文件
    :param save_image: 保存的二维码图片
    :param option: 二维码配置
    :return: pil图片
    """
    default_option = {
        # 二维码的每个box像素块的大小是多少像素
        'box_size': 10,
        # 二维码与图片的边缘的距离是多少个box
        'border': 2,
        # 前景色
        'fill_color': 'black',
        # 背景色
        'back_color': 'white',
        # 图片像素大小
        'pixel_size': 290,
        # 图片类型
        'kind': 'PNG',
    }
    if not option:
        option = default_option
    qr = qrcode.QRCode(
        # version值范围[1-40]整数，1表示21x21矩阵，值为None，qr.make(fit=True),则系统自动调整大小
        version=4,  # 直观的感受是二维码中像素的密集程度，数越大，密集程度越高
        error_correction=qrcode.constants.ERROR_CORRECT_M,  # 容错率 7%，15%，25%，30%
        **option
    )
    # 二维码内容
    qr.add_data(data_str)
    # 图片中的二维码大小自适应，以保证二维码内容能完整绘制
    qr.make(fit=True)
    # 前景、背景色
    pil_img = qr.make_image(fill_color="blue", back_color="white")
    pil_img.save(save_image)
    return pil_img


def make_qrcode_image(data_str: str, save_image: str) -> Image:
    """创建二维码图片"""
    pil_img = qrcode.make(data_str)
    pil_img.save(save_image)
    return pil_img


def make_qrcode_image_base64(data_str: str, border: int = 2) -> str:
    """创建二维码图片"""
    pil_img = qrcode.make(data_str, border=border)
    # 创建一个BytesIO对象，用于临时存储图像数据
    image_data = io.BytesIO()
    # 将图像保存到BytesIO对象中，格式为JPEG
    pil_img.save(image_data)
    # 将BytesIO对象的内容转换为字节串
    image_data_bytes = image_data.getvalue()
    # 将图像数据编码为Base64字符串
    return 'data:image/png;base64,' + base64.b64encode(image_data_bytes).decode('utf-8')


if __name__ == '__main__':
    print(make_qrcode_image_base64('https://127.0.0.1:8080/reader/12345', border=1))



import json
import os.path
from typing import Any, Optional, Mapping, Sequence

from starlette.background import BackgroundTask
from starlette.requests import Request
from starlette.responses import JSONResponse, FileResponse, Response, RedirectResponse

from afeng_tools.fastapi_tool import fastapi_settings
from afeng_tools.fastapi_tool.core.fastapi_enums import FastapiConfigKeyEnum
from afeng_tools.sqlalchemy_tool.core import sqlalchemy_model_utils
from afeng_tools.sqlalchemy_tool.core.sqlalchemy_base_model import Model
from afeng_tools.web_tool import request_tools


def resp_404(message: str = '页面没找到！', request: Request = None, context_data: dict = None):
    if request and not request_tools.is_json(request.headers):
        is_json_api = fastapi_settings.get_config(FastapiConfigKeyEnum.is_json_api)
        if is_json_api:
            return JSONResponse(
                status_code=404,
                content={"message": message, 'error_no': 404},
            )
        is_mobile = request_tools.is_mobile(request.headers.get('user-agent'))
        if context_data is None:
            context_data_func = fastapi_settings.get_config(FastapiConfigKeyEnum.error404_context_data_func)
            if context_data_func:
                context_data = context_data_func(message=message, is_mobile=is_mobile)
            else:
                context_data = {
                    "title": f'404错误页面',
                    'message': message
                }
        from afeng_tools.fastapi_tool.fastapi_jinja2_tools import create_template_response
        return create_template_response(request=request,
                                        template_file=('mobile' if is_mobile else 'pc') + '/views/error/404.html',
                                        context=context_data)
    return JSONResponse(
        status_code=404,
        content={"message": message, 'error_no': 404},
    )


def resp_501(message: str, request: Request = None, context_data: dict = None):
    if request and not request_tools.is_json(request.headers):
        is_json_api = fastapi_settings.get_config(FastapiConfigKeyEnum.is_json_api)
        if is_json_api:
            return JSONResponse(
                status_code=501,
                content={"message": message, 'error_no': 501},
            )
        is_mobile = request_tools.is_mobile(request.headers.get('user-agent'))
        if context_data is None:
            context_data_func = fastapi_settings.get_config(FastapiConfigKeyEnum.error501_context_data_func)
            if context_data_func:
                context_data = context_data_func(message=message, is_mobile=is_mobile)
            else:
                context_data = {
                    "title": f'操作失败页面',
                    'message': message
                }
        from afeng_tools.fastapi_tool.fastapi_jinja2_tools import create_template_response
        return create_template_response(request=request,
                                        template_file=('mobile' if is_mobile else 'pc') + '/views/error/501.html',
                                        context=context_data)
    return JSONResponse(
        status_code=501,
        content={"message": message, 'error_no': 501},
    )


def resp_422(message: str | Sequence):
    return JSONResponse(
        status_code=422,
        content={"message": message, 'error_no': 422},
    )


def resp_500(message: str = '服务器内部错误！', request: Request = None, context_data: dict = None):
    if request and not request_tools.is_json(request.headers):
        is_json_api = fastapi_settings.get_config(FastapiConfigKeyEnum.is_json_api)
        if is_json_api:
            return JSONResponse(
                status_code=500,
                content={"message": message, 'error_no': 500},
            )
        is_mobile = request_tools.is_mobile(request.headers.get('user-agent'))
        if context_data is None:
            context_data_func = fastapi_settings.get_config(FastapiConfigKeyEnum.error500_context_data_func)
            if context_data_func:
                context_data = context_data_func(message=message, is_mobile=is_mobile)
            else:
                context_data = {
                    "title": f'500错误页面',
                    'message': message
                }
        from afeng_tools.fastapi_tool.fastapi_jinja2_tools import create_template_response
        return create_template_response(request=request,
                                        template_file=('mobile' if is_mobile else 'pc') + '/views/error/500.html',
                                        context=context_data)
    return JSONResponse(
        status_code=500,
        content={"message": message, 'error_no': 500},
    )


def resp_json(data: Any = None, error_no: int = 0, message: str = 'success'):
    if isinstance(data, Model) or (data and isinstance(data, list) and len(data) > 0 and isinstance(data[0], Model)):
        data = json.loads(sqlalchemy_model_utils.to_json(data))
    return JSONResponse(
        status_code=200,
        content={"message": message, 'error_no': error_no, 'data': data},
    )


def resp_file(file_path: str, file_name: str = None, download_flag: bool = False,
              context_data: dict = None) -> Response:
    """响应文件"""
    if not os.path.exists(file_path):
        return resp_404('资源不存在！', context_data=context_data)
    response = FileResponse(file_path)
    with open(file_path, "rb") as file:
        if download_flag:
            if file_name is None:
                file_name = os.path.split(file_path)[1]
            response.headers["Content-Disposition"] = f"attachment; filename={file_name}"
        response.body = file.read()
        return response


def redirect(target_url: str, status_code: int = 307,
             headers: Optional[Mapping[str, str]] = None,
             background: Optional[BackgroundTask] = None, ) -> RedirectResponse:
    """重定向"""
    return RedirectResponse(target_url, status_code=status_code, headers=headers, background=background)



"""
图片和base64工具
"""
import base64
import re

from afeng_tools.http_tool import http_mimetype_tools


def base64_to_image(img_base64_data: str, save_image_file: str) -> str:
    """
    base64图片数据转换为图片
    :param img_base64_data: base64图片数据
    :param save_image_file: 保存的图片文件，如：output/test.png
    :return: save_image 保存的图片文件
    """
    img_data = base64.b64decode(re.sub('^data:image/.+;base64,', '', img_base64_data))
    with open(save_image_file, mode='wb') as file:
        file.write(img_data)
    return save_image_file


def image_to_base64(image_file: str) -> str:
    """
    图片转为base64
    :param image_file: 图片文件
    :return: base64图片字符串
    """
    with open(image_file, 'rb') as file:
        image_data = file.read()
        base64_data = base64.b64encode(image_data).decode('utf-8')
    mimetype = http_mimetype_tools.get_mimetype(image_file)
    return f'data:{mimetype[0]};base64,{base64_data}'




"""
响应工具
"""
import json
from typing import Any
from starlette.responses import JSONResponse, FileResponse, Response, RedirectResponse

from afeng_tools.sqlalchemy_tool.core import sqlalchemy_model_utils
from afeng_tools.sqlalchemy_tool.core.sqlalchemy_base_model import Model
from afeng_tools.web_tool.core.web_common_models import ResponseModel


def create_json_response_data(data: Any = None, error_no: int = 0, message: str = 'success') -> ResponseModel:
    if isinstance(data, Model) or (data and isinstance(data, list) and len(data) > 0 and isinstance(data[0], Model)):
        data = json.loads(sqlalchemy_model_utils.to_json(data))
    return ResponseModel(
        error_no=error_no,
        message=message,
        data=data
    )


def create_json_response(response_model: ResponseModel) -> JSONResponse:
    return JSONResponse(
        status_code=200,
        content=response_model.model_dump()
    )


def json_response(data: Any = None, error_no: int = 0, message: str = 'success') -> JSONResponse:
    response_model = create_json_response_data(data=data, error_no=error_no, message=message)
    return create_json_response(response_model=response_model)





"""
FastAPI路由工具
"""
import importlib
import os

from fastapi import APIRouter


def create_router(prefix: str, tags: list[str]) -> APIRouter:
    """创建路由"""
    router = APIRouter(prefix=prefix, tags=tags)
    return router


def auto_load_routers(web_root_path: str, web_root_name: str = None,
                      view_file_name: str = 'views', router_name='router') -> list[APIRouter]:
    """自动加载web.{app}.views文件中的路由"""
    router_list = []
    if web_root_name is None:
        web_root_name = os.path.split(web_root_path)[1]
    for app_name in os.listdir(web_root_path):
        if os.path.isdir(os.path.join(web_root_path, app_name)) and app_name != '__pycache__' and app_name != 'admin':
            if os.path.exists(os.path.join(web_root_path, app_name, view_file_name+'.py')):
                web_app_views = importlib.import_module(f'{web_root_name}.{app_name}.{view_file_name}')
                router_list.append(web_app_views.__getattribute__(router_name))
    return router_list



import enum
import json
from typing import Type, Any, Optional, Iterable

from fastapi import Form
from pydantic import BaseModel
from sqlalchemy import String, func, Text, Boolean, ARRAY
from starlette.requests import Request
from starlette.responses import JSONResponse, HTMLResponse
from fastapi.encoders import jsonable_encoder

from afeng_tools.fastapi_tool.fastapi_response_tools import resp_json
from afeng_tools.id_code_tool import id_code_tools
from afeng_tools.layui_tool import layui_template_tools
from afeng_tools.layui_tool.core.layui_models import LayUiFormItem, LayUiTableCol, LayUiTableActionItem, LayUiTableData
from afeng_tools.pydantic_tool.model.common_models import EnumItem

from afeng_tools.sqlalchemy_tool.core import orm_model_tools
from afeng_tools.sqlalchemy_tool.core.sqlalchemy_base_model import Model
from afeng_tools.sqlalchemy_tool.crdu import base_crdu


class AdminViewsTemplate:
    def __init__(self, module_code: str, orm_model_class: Type[Model], search_form_item_list: list[LayUiFormItem]):
        """
        Admin管理Views模板
        :param module_code: 模块编码
        :param orm_model_class: orm模型类
        :param search_form_item_list: 用于查询的表单项列表
        """
        self.module_code = module_code
        self.orm_model_class = orm_model_class
        self.search_form_item_list = search_form_item_list

    def _create_form_item_list(self, old_data=None):
        form_item_list = []
        for c in self.orm_model_class.__table__.columns:

            if issubclass(c.type.python_type, enum.Enum):
                tmp_form_item = LayUiFormItem(title=c.comment, name=c.name, type='select',
                                              select_list=[((tmp.value if isinstance(tmp.value, str) else tmp.name), tmp.name, False, False) for tmp in
                                                           c.type.python_type])
            elif isinstance(c.type, Boolean):
                tmp_form_item = LayUiFormItem(title=c.comment, name=c.name, type='switch',
                                              switch_item=('1', '开启|关闭', False, False))
            elif isinstance(c.type, ARRAY):
                tmp_form_item = LayUiFormItem(title=c.comment, name=c.name, type='array')
            elif issubclass(c.type.python_type, int):
                tmp_form_item = LayUiFormItem(title=c.comment, name=c.name, type='number')
            elif isinstance(c.type, Text):
                tmp_form_item = LayUiFormItem(title=c.comment, name=c.name, type='textarea')
            else:
                tmp_form_item = LayUiFormItem(title=c.comment, name=c.name)
            if old_data is not None:
                if issubclass(c.type.python_type, enum.Enum):
                    tmp_select_list = []
                    for tmp_value, tmp_title, tmp_selected, tmp_disabled in tmp_form_item.select_list:
                        old_data_field_value = getattr(old_data, c.name)
                        if isinstance(old_data_field_value, enum.Enum):
                            old_data_field_value = old_data_field_value.value.value
                        if tmp_value == old_data_field_value:
                            tmp_selected = True
                        tmp_select_list.append((tmp_value, tmp_title, tmp_selected, tmp_disabled))
                    tmp_form_item.select_list = tmp_select_list
                elif isinstance(c.type, Boolean):
                    tmp_value, tmp_title, tmp_checked, tmp_disabled = tmp_form_item.switch_item
                    if getattr(old_data, c.name):
                        tmp_checked = True
                    tmp_form_item.switch_item = (tmp_value, tmp_title, tmp_checked, tmp_disabled)
                elif isinstance(c.type, ARRAY):
                    tmp_value = getattr(old_data, c.name)
                    if tmp_value and isinstance(tmp_value, Iterable):
                        tmp_form_item.default = ','.join([str(tmp) for tmp in tmp_value])
                else:
                    print(type(old_data), c.name)
                    tmp_value = getattr(old_data, c.name)
                    if tmp_value is not None:
                        print(type(tmp_value), c.name)
                        tmp_form_item.default = getattr(old_data, c.name)
            form_item_list.append(tmp_form_item)
        return form_item_list

    async def index_page(self) -> HTMLResponse:
        """列表页面"""
        model_title = self.orm_model_class.__doc__
        col_list = [LayUiTableCol(field=c.name, title=c.comment, sort=True, is_enum=issubclass(c.type.python_type, enum.Enum)) for c in
                    self.orm_model_class.__table__.columns]
        return HTMLResponse(layui_template_tools.render_table_template(title=f'{model_title}-管理',
                                                                       search_form_item_list=self.search_form_item_list,
                                                                       data_url=f'/admin/api/{self.module_code}/list',
                                                                       col_list=col_list,
                                                                       add=True,
                                                                       add_html_url=f'/admin/{self.module_code}/add',
                                                                       edit=True,
                                                                       edit_html_url=f'/admin/{self.module_code}/update',
                                                                       delete=True,
                                                                       delete_url=f'/admin/api/{self.module_code}/delete',
                                                                       more_delete=True,
                                                                       more_delete_url=f'/admin/api/{self.module_code}/more_delete',
                                                                       action_list=[
                                                                           LayUiTableActionItem(title='展示',
                                                                                                url=f'/admin/{self.module_code}/show')
                                                                       ]
                                                                       ))

    async def add_page(self) -> HTMLResponse:
        """添加页面"""
        model_title = self.orm_model_class.__doc__
        return HTMLResponse(layui_template_tools.render_add_template(title=f'添加-{model_title}',
                                                                     form_item_list=self._create_form_item_list(),
                                                                     add_url=f'/admin/api/{self.module_code}/add'))

    async def update_page(self, request: Request, id_value: int) -> HTMLResponse:
        """修改页面"""
        model_title = self.orm_model_class.__doc__
        old_data = base_crdu.query_one(self.orm_model_class,
                                       self.orm_model_class.id == id_value)
        return HTMLResponse(layui_template_tools.render_edit_template(title=f'修改-{model_title}',
                                                                      form_item_list=self._create_form_item_list(
                                                                          old_data),
                                                                      edit_url=f'/admin/api/{self.module_code}/update'))

    async def show_page(self, request: Request, id_value: int) -> HTMLResponse:
        """展示页面"""
        model_title = self.orm_model_class.__doc__
        old_data = base_crdu.query_one(self.orm_model_class,
                                       self.orm_model_class.id == id_value)
        if old_data:
            form_item_list = [
                LayUiFormItem(title=c.comment, name=c.name, default=getattr(old_data, c.name), readonly=True)
                for c in self.orm_model_class.__table__.columns]
        else:
            form_item_list = [LayUiFormItem(title=c.comment, name=c.name)
                              for c in self.orm_model_class.__table__.columns]
        return HTMLResponse(layui_template_tools.render_show_template(title=f'展示-{model_title}',
                                                                      form_item_list=form_item_list))

    async def add_data(self, request: Request, form_data: BaseModel) -> JSONResponse:
        """添加数据"""
        po = orm_model_tools.py_model_to_orm_model(form_data, self.orm_model_class)
        po = base_crdu.add(po)
        self.add_data_after_handle(po)
        return resp_json(po)

    async def update_data(self, request: Request, form_data: BaseModel) -> JSONResponse:
        """更新数据"""
        po = orm_model_tools.py_model_to_orm_model(form_data, self.orm_model_class)
        po = base_crdu.update(po)
        self.update_data_after_handle(po)
        return resp_json(po)

    async def delete_data(self, request: Request, id_value: int) -> JSONResponse:
        """删除数据"""
        old_po = base_crdu.query_one(self.orm_model_class, self.orm_model_class.id == id_value)
        base_crdu.delete_by_ids(self.orm_model_class, [id_value])
        self.delete_data_after_handle(old_po)
        return resp_json('success')

    async def more_delete_data(self, request: Request, ids: str = Form()):
        id_list = json.loads(ids)
        old_po_list = base_crdu.query_all(self.orm_model_class, self.orm_model_class.id.in_(id_list))
        base_crdu.delete_by_ids(self.orm_model_class, id_list)
        for old_po in old_po_list:
            self.delete_data_after_handle(old_po)
        return resp_json('success')

    async def get_data(self, request: Request, id_value: int) -> JSONResponse:
        """查询单条数据"""
        return resp_json(base_crdu.query_one(self.orm_model_class, self.orm_model_class.id == id_value))

    async def list_data(self, request: Request, page: Optional[int] = 1, limit: Optional[int] = 15,
                        searchParams: Optional[str] = None):
        """列出所有数据"""
        search_params = None
        if searchParams:
            search_params = json.loads(searchParams)
        query = base_crdu.create_query(self.orm_model_class)
        if search_params:
            search_field_list = [tmp.name for tmp in self.search_form_item_list]
            for tmp_key, tmp_value in search_params.items():
                if tmp_key in search_field_list and tmp_value is not None and len(tmp_value) > 0:
                    query = query.filter(
                        func.cast(getattr(self.orm_model_class, tmp_key), String).ilike(f'%{tmp_value}%'))
        query = query.order_by(self.orm_model_class.add_time.desc())
        return jsonable_encoder(LayUiTableData(
            total=query.count(),
            data_list=query.offset((page - 1) * limit).limit(limit).all()
        ).model_dump())

    def add_data_after_handle(self, po_model: Any):
        """添加数据后处理"""
        code_column = [c for c in self.orm_model_class.__table__.columns if c.name == 'code']
        if code_column:
            setattr(po_model, 'code', id_code_tools.get_code_by_id(po_model.id))
            base_crdu.update(po_model)

    def update_data_after_handle(self, po_model: Any):
        """更新数据后处理"""
        pass

    def delete_data_after_handle(self, po_model: Any):
        """删除数据后处理"""
        pass



import uvicorn
from config import settings
from afeng_tools.application_tool import settings_tools
from afeng_tools.application_tool.settings_enum import SettingsKeyEnum
from web.apps import app

if __name__ == '__main__':
    http_host = settings_tools.get_config('server.host')
    http_port = settings_tools.get_config('server.port')
    # 实现类似命令的效果：uvicorn main:app --reload  --host 0.0.0.0
    if settings_tools.get_config(SettingsKeyEnum.app_is_debug):
        uvicorn.run('main:app', host=http_host, port=http_port, log_level="debug", reload=True, workers=4)
    else:
        uvicorn.run('main:app', host=http_host, port=http_port, log_level="info", workers=8)



from typing import Type

from afeng_tools.fastapi_tool.admin.admin_views_template import AdminViewsTemplate
from afeng_tools.layui_tool.core.layui_models import LayUiFormItem
from afeng_tools.sqlalchemy_tool.core.sqlalchemy_base_model import Model


class AdminRouterModel:
    """管理路由模型"""

    def __init__(self, module_code: str, tag_list: list[str], orm_model_class: Type[Model], search_form_item_list: list[LayUiFormItem],
                 admin_views_template: AdminViewsTemplate = None,
                 admin_views_template_class: Type[AdminViewsTemplate] = None):
        self.module_code = module_code
        self.tag_list = tag_list
        self.orm_model_class = orm_model_class
        self.search_form_item_list = search_form_item_list
        if admin_views_template is None:
            if admin_views_template_class:
                self.admin_views_template = admin_views_template_class(module_code=self.module_code,
                                                                       orm_model_class=self.orm_model_class,
                                                                       search_form_item_list=self.search_form_item_list)
            else:
                self.admin_views_template = AdminViewsTemplate(module_code=self.module_code,
                                                               orm_model_class=self.orm_model_class,
                                                               search_form_item_list=self.search_form_item_list)
        else:
            self.admin_views_template = admin_views_template



"""
类型转换工具
"""
from array import array
from datetime import datetime, date
from enum import Enum
from typing import Any, Type

from _decimal import Decimal

from afeng_tools.pydantic_tool.model.common_models import EnumItem

import typing


def convert_value_to_type(value: Any, to_type: Type[Any], date_time_format: str = '%Y-%m-%d %H:%M:%S',
                          date_format: str = '%Y-%m-%d') -> Any:
    """
    转换值为某个类型
    :param value: 原值
    :param to_type: 需要的类型
    :param date_time_format: 日期时间格式
    :param date_format: 日期格式
    :return: 转换后的值
    """
    if to_type.__name__ == 'Optional':
        to_type = to_type.__dict__.get('__args__')[0]
    if to_type is None:
        return value
    elif isinstance(value, str):
        value = value.strip()
    if isinstance(value, str) and len(value) == 0:
        return None
    if isinstance(value, to_type):
        return value
    elif isinstance(value, datetime) and issubclass(to_type, str):
        return value.strftime(date_time_format)
    elif isinstance(value, str) and issubclass(datetime, to_type):
        return datetime.strptime(value, date_time_format)
    elif issubclass(str, to_type):
        return str(value)
    elif issubclass(int, to_type):
        return int(value)
    elif issubclass(float, to_type):
        return float(value)
    elif issubclass(bool, to_type) and isinstance(value, str):
        return 'true' == value.lower() or '1' == value
    elif isinstance(value, date):
        return value.strftime(date_format)
    elif isinstance(value, bytes) and issubclass(to_type, str):
        return str(value, encoding='utf-8')
    elif isinstance(value, array):
        return value.tolist()
    elif isinstance(value, Decimal) and issubclass(to_type, str):
        return str(value)
    elif isinstance(value, Enum) and not issubclass(to_type, Enum):
        if isinstance(value.value, str):
            return value.value
        elif isinstance(value.value, EnumItem) and value.value.value:
            return value.value.value
        return value.name
    elif not isinstance(value, Enum) and issubclass(to_type, Enum):
        enum_list = [tmp for tmp in to_type if tmp.name == value or tmp.value == value or tmp.value.value == value]
        if enum_list:
            return enum_list[0]
        else:
            return value
    else:
        return str(value)



"""
百度网盘音频文件工具
"""
import requests

from afeng_tools.baidu_pan_tool.core.baidu_pan_models import ListAudioCategoryResult, AudioCategoryInfo, ListAudioResult, AudioInfo
from afeng_tools.http_tool import http_params_tools
from afeng_tools.log_tool.loguru_tools import get_logger, log_error

logger = get_logger()


def list_audio_category(access_token: str, page_num: int = 1, page_size: int = 20) -> ListAudioCategoryResult:
    """
    获取播单列表: 本接口用于查询用户下的所有播单。
    :param access_token: Access Token
    :param page_num: 当前页数 ，默认1
    :param page_size: 每页返回的播单数目，默认20
    :return: ListAudioCategoryResult
    """
    params = {
        'method': 'list',
        'access_token': access_token,
        'page': page_num,
        'psize': page_size
    }
    headers = {
        'Host': 'pan.baidu.com'
    }
    try:
        response = requests.get('http://pan.baidu.com/rest/2.0/xpan/broadcast/list', params=params, headers=headers)
        response.encoding = 'utf-8'
        if response.status_code == 200:
            api_response = response.json()
            if api_response['errno'] == 0:
                result = ListAudioCategoryResult(errno=api_response.get('errno'),
                                                 request_id=api_response.get('request_id'),
                                                 show_msg=api_response.get('show_msg'),
                                                 newno=api_response.get('newno'),
                                                 has_more=api_response.get('has_more'))
                if api_response.get('list'):
                    result.data_list = [AudioCategoryInfo(**tmp) for tmp in api_response.get('list')]
                return result
            else:
                log_error(logger, f"[BaiduPan]获取播单列表失败:[{api_response}]")
        else:
            log_error(logger, f"[BaiduPan]获取播单列表失败:[{response.status_code}]{response.text}")
    except Exception as e:
        log_error(logger, f"[BaiduPan]获取播单列表失败", e)


def list_audio(access_token: str, mb_id: int, show_meta: int = 0,
               page_num: int = 1, page_size: int = 20) -> ListAudioResult:
    """
    获取播单下文件列表
    :param access_token: 本接口用于根据播单id获取播单内文件列表。
    :param mb_id: 播单id, 该值通过list_audio_category()获取播单列表获取
    :param show_meta: showmeta=1，可展示文件详细信
    :param page_num: 当前页数，默认1
    :param page_size: 每页返回的播单数目，默认20
    :return:
    """
    params = {
        'access_token': access_token,
        'mb_id': mb_id,
        'showmeta': show_meta,
        'page': page_num,
        'psize': page_size
    }
    headers = {
        'Host': 'pan.baidu.com'
    }
    try:
        response = requests.get('http://pan.baidu.com/rest/2.0/xpan/broadcast/filelist', params=params, headers=headers)
        response.encoding = 'utf-8'
        if response.status_code == 200:
            api_response = response.json()
            if api_response['errno'] == 0:
                result = ListAudioResult(errno=api_response.get('errno'),
                                         request_id=api_response.get('request_id'),
                                         show_msg=api_response.get('show_msg'),
                                         newno=api_response.get('newno'),
                                         has_more=api_response.get('has_more'))
                if api_response.get('list'):
                    result.data_list = [AudioInfo(**tmp) for tmp in api_response.get('list')]
                return result
            else:
                log_error(logger, f"[BaiduPan]获取播单[{mb_id}]列表失败:[{api_response}]")
        else:
            log_error(logger, f"[BaiduPan]获取播单[{mb_id}]列表失败:[{response.status_code}]{response.text}")
    except Exception as e:
        log_error(logger, f"[BaiduPan]获取播单[{mb_id}]列表失败", e)


def _get_audio_type(pan_path: str):
    """获取音频分片格式"""
    if pan_path.endswith('.mp3'):
        return 'M3U8_MP3_128'
    else:
        return 'M3U8_HLS_MP3_128'


def get_audio_m3u8_url(access_token: str, pan_path: str) -> tuple[str, dict[str, str]]:
    """
    获取音频播放的m3u8地址(可以使用百度web播放器播放：https://cloud.baidu.com/doc/MCT/s/zjwvz4w4z)
    :param access_token:
    :param pan_path:
    :return: (m3u8_url,headers)
    """
    params = {
        'method': 'streaming',
        'access_token': access_token,
        'path': pan_path,
        'type': _get_audio_type(pan_path)
    }
    headers = {
        'User-Agent': 'xpanvideo;netdisk;iPhone13;ios-iphone;15.1;ts',
        'Host': 'pan.baidu.com'
    }
    m3u8_url = 'http://pan.baidu.com/rest/2.0/xpan/file?' + http_params_tools.url_encode_params(params)
    return m3u8_url, headers



from afeng_tools.aliyun_pan_tool.aliyun_pan_share_tools import get_share_info_by_link, list_share_root_files, get_share_token
from afeng_tools.aliyun_pan_tool.aliyun_pan_tools import get_ali_api


def list_share_root_files_demo():
    ali_api = get_ali_api()
    result = get_share_info_by_link(ali_api, share_msg='https://www.aliyundrive.com/s/1GmJaHi5aMk/folder/625abafa180f8e98698c4ce49c9b04809d3202fc')
    print(result)
    share_token = get_share_token(ali_api, result.share_id)
    root_file_list = list_share_root_files(ali_api, share_token)
    for tmp_file in root_file_list:
        print(tmp_file)

def download_share_file_demo():
    pass

if __name__ == '__main__':
    list_share_root_files_demo()


"""
日志枚举
"""
from enum import Enum


class LoggerConfigKeyEnum(Enum):
    info_file = 'info_file'
    info_rotation = 'info_rotation'
    error_file = 'error_file'
    error_rotation = 'error_rotation'



"""
阿里云盘工具
"""
import os
from typing import List, Callable

from aligo import Aligo, BaseUser, BaseFile, EMailConfig, set_config_folder, CreateFileResponse, GetFileRequest
from aligo.types.Enum import CheckNameMode

from afeng_tools.encryption_tool import hashlib_tools
from afeng_tools.file_tool.tmp_file_tools import get_user_tmp_dir
from afeng_tools.serialization_tool import pickle_tools


def get_ali_api_by_email(receive_email: str, send_email: str, send_password: str, email_host: str,
                         email_port: int) -> Aligo:
    """
    获取api接口：发送登录二维码到邮箱(建议将邮箱绑定到微信，这样能实时收到提醒，登录过期后也可以第一时间收到登录请求。)
    :param receive_email:  接收登录邮件的邮箱地址
    :param send_email: 发送邮件的邮箱
    :param send_password: 发送邮件的密码
    :param email_host: 发送邮件的主机
    :param email_port: 发送邮件的端口
    :return: Aligo
    """
    email_config = EMailConfig(
        email=receive_email,
        # 自配邮箱
        user=send_email,
        password=send_password,
        host=email_host,
        port=email_port,
    )
    return Aligo(email=email_config)


def get_ali_api_by_web(port: int, use_aria2: bool = False) -> Aligo:
    """获取api接口：打开浏览器访问 http://<YOUR_IP>:<port> 网页扫码登录"""
    return Aligo(port=port, use_aria2=use_aria2)


def get_ali_api(name: str = None, config_path: str = None, use_aria2: bool = False) -> Aligo:
    """
    获取api接口：第一次使用，会弹出二维码，供扫描登录
    :param name: 配置文件名，如：name='一号服务器'， 会创建 <用户目录>/.alig/一号服务器.json 配置文件
    :param config_path: 配置文件目录，默认是 <用户目录>/.alig
    :param use_aria2: 是否使用 aria2 下载
    :return:
    """
    if config_path:
        set_config_folder(config_path)
    if name:
        return Aligo(name=name, use_aria2=use_aria2)
    return Aligo(use_aria2=use_aria2)


def get_user_info(ali_api: Aligo) -> BaseUser:
    """获取用户信息"""
    return ali_api.get_user()


def list_file(ali_api: Aligo, parent_file_id: str = 'root') -> List[BaseFile]:
    """列出路径下的文件"""
    return ali_api.get_file_list(parent_file_id=parent_file_id)


def list_path(ali_api: Aligo, pan_path: str) -> List[BaseFile] | None:
    """列出路径下的文件"""
    pan_file = get_file_info(ali_api, pan_path)
    if pan_file:
        return list_file(ali_api, parent_file_id=pan_file.file_id)


def list_all_file(ali_api: Aligo, pan_path: str,
                  root_cache_path: str = None, callback_func: Callable = None) -> list[tuple[str, BaseFile]]:
    """
    递归列出目录下文件
    :param ali_api: Aligo
    :param pan_path: 要递归列出的网盘路径
    :param root_cache_path: 当是内部递归调用时，传入缓存的根路径
    :param callback_func: 获取到文件信息后的回调函数, 有两个参数：pan_path:str, pan_file:BaseFile
    :return: list[tuple[文件路径, BaseFile]]
    """
    ali_file_list = list_path(ali_api, pan_path)
    if ali_file_list:
        file_list = []
        is_root: bool = False
        if root_cache_path is None:
            is_root = True
            root_cache_path = os.path.join(get_user_tmp_dir(), hashlib_tools.calc_md5(pan_path))
            os.makedirs(root_cache_path, exist_ok=True)
        for tmp in ali_file_list:
            tmp_file_name = pan_path + '/' + tmp.name
            print(f'[{tmp.type}]{tmp_file_name}')
            if tmp.type == 'folder':
                tmp_cache = os.path.join(root_cache_path, tmp.file_id)
                if os.path.exists(tmp_cache):
                    child_list = pickle_tools.parse_to_obj(tmp_cache)
                    if child_list and callback_func and isinstance(callback_func, Callable):
                        for temp_child_file_name, temp_child_file in child_list:
                            callback_func(temp_child_file_name, temp_child_file)
                else:
                    child_list = list_all_file(ali_api, tmp_file_name,
                                               root_cache_path=root_cache_path,
                                               callback_func=callback_func)
                if child_list:
                    file_list.extend(child_list)
                    if not os.path.exists(tmp_cache):
                        pickle_tools.save_to_file(child_list, tmp_cache)
            else:
                file_list.append((tmp_file_name, tmp))
                if callback_func and isinstance(callback_func, Callable):
                    callback_func(tmp_file_name, tmp)
        if is_root:
            # 先注释了删除
            # shutil.rmtree(root_cache_path)
            pass
        return file_list


def get_file_info(ali_api: Aligo, pan_path: str, parent_file_id: str = 'root') -> BaseFile | None:
    """
    通过文件路径获取文件信息
    :param ali_api: Aligo
    :param pan_path: 网盘文件路径
    :param parent_file_id: 父id
    :return: 文件基础信息
    """
    pan_path = pan_path.strip('/')
    path_list = pan_path.split('/')
    if len(path_list) == 1:
        for tmp in list_file(ali_api, parent_file_id=parent_file_id):
            if tmp.name == path_list[0]:
                return tmp
        return
    else:
        parent_folder = ali_api.get_folder_by_path('/'.join(path_list[0:-1]), parent_file_id=parent_file_id,
                                                   create_folder=False)
        if parent_folder:
            parent_file_id = parent_folder.file_id
            folder = ali_api.get_folder_by_path(path_list[-1], parent_file_id=parent_file_id, create_folder=False)
            if folder:
                return folder
            file_list = ali_api.get_file_list(parent_file_id=parent_file_id, type='file')
            for file in file_list:
                if path_list[-1] == file.name:
                    return file


def get_file_by_id(ali_api: Aligo, file_id: str, drive_id: str = None) -> BaseFile:
    """
    通过文件id获取文件信息
    :param ali_api: Aligo
    :param file_id: 文件id
    :param drive_id: Optional[str] 指定网盘id, 默认为 None
    :return: 文件基础信息
    """
    return ali_api._core_get_file(GetFileRequest(file_id=file_id, drive_id=drive_id))


def create_folder(ali_api: Aligo, name: str, parent_file_id: str = 'root', drive_id: str = None,
                  check_name_mode: CheckNameMode = 'auto_rename') -> CreateFileResponse:
    """
    创建文件夹
    :param ali_api: Aligo
    :param name: [str] 文件夹名
    :param parent_file_id: Optional[str] 父文件夹id, 默认为 'root'
    :param drive_id: Optional[str] 指定网盘id, 默认为 None
    :param check_name_mode: Optional[CheckNameMode] 检查文件名模式, 默认为 'auto_rename'
    :return: [CreateFileResponse]
    """
    return ali_api.create_folder(name=name, parent_file_id=parent_file_id, drive_id=drive_id,
                                 check_name_mode=check_name_mode)


def upload_files(ali_api: Aligo, local_file_paths: List[str], pan_path: str = None, drive_id: str = None,
                 check_name_mode: CheckNameMode = "auto_rename") -> List[BaseFile]:
    """
    批量上传文件
    :param ali_api: Aligo
    :param local_file_paths: [List[str]] 文件路径列表, 如：['/Users/aligo/Desktop/test1.txt', '/Users/aligo/Desktop/test2.txt']
    :param pan_path: 上传到的网盘路径, 如：'我的资源/1080p'
    :param drive_id: Optional[str] 指定网盘id, 默认为 None
    :param check_name_mode: Optional[CheckNameMode] 检查文件名模式, 默认为 'auto_rename'
    :return: [List[BaseFile]]
    """
    if pan_path is None:
        parent_file_id = 'root'
    else:
        parent_file_id = ali_api.get_folder_by_path(pan_path, create_folder=True).file_id
    return ali_api.upload_files(file_paths=local_file_paths, parent_file_id=parent_file_id, drive_id=drive_id,
                                check_name_mode=check_name_mode)


def upload_folder(ali_api: Aligo, local_folder_path: str, pan_path: str = None, drive_id: str = None,
                  check_name_mode: CheckNameMode = "auto_rename", folder_check_name_mode: CheckNameMode = 'refuse',
                  file_filter: Callable[[os.DirEntry], bool] = lambda x: False) -> List:
    """
    上传文件夹
    :param ali_api: Aligo
    :param local_folder_path: [str] 文件夹路径
    :param pan_path: 上传到的网盘路径, 如：'我的资源/1080p'
    :param drive_id: [str] 指定网盘id, 默认为 None, 如果为 None, 则使用默认网盘
    :param check_name_mode: [CheckNameMode] 检查文件名模式, 默认为 'auto_rename'
    :param folder_check_name_mode: [CheckNameMode] 检查文件夹名模式, 默认为 'refuse'
    :param file_filter: 文件过滤函
    :return:
    """
    if pan_path is None:
        parent_file_id = 'root'
    else:
        parent_file_id = ali_api.get_folder_by_path(pan_path, create_folder=True).file_id
    return ali_api.upload_folder(local_folder_path, parent_file_id=parent_file_id, drive_id=drive_id,
                                 check_name_mode=check_name_mode, folder_check_name_mode=folder_check_name_mode,
                                 file_filter=file_filter)


def download_file_by_path(ali_api: Aligo, pan_path: str, local_path: str,
                          pan_file_filter: Callable[[BaseFile], bool] = lambda x: False) -> str:
    """
    下载文件/文件夹
    :param ali_api: Aligo
    :param pan_path: 网盘路径，如：我的资源/音乐
    :param local_path: 本地存储路径，如：D:/阿里云盘
    :param pan_file_filter: 文件过滤函数
    :return:
    """
    file = ali_api.get_file_by_path(pan_path)
    return download_file(ali_api, pan_file=file, local_path=local_path, pan_file_filter=pan_file_filter)


def download_file(ali_api: Aligo, pan_file: BaseFile, local_path: str,
                  pan_file_filter: Callable[[BaseFile], bool] = lambda x: False) -> str:
    """
    下载文件/文件夹
    :param ali_api: Aligo
    :param pan_file: 网盘文件
    :param local_path: 本地存储路径
    :param pan_file_filter: 文件过滤函数
    :return: [str] 本地文件路径
    """
    if pan_file.type == 'file':
        local_file = ali_api.download_file(file=pan_file, local_folder=local_path)
        if os.path.exists(local_file):
            if os.stat(local_file).st_size == 350:
                os.remove(local_file)
                local_file = ali_api.download_file(file_id=pan_file.file_id, local_folder=local_path)
            return local_file
    else:
        return ali_api.download_folder(folder_file_id=pan_file.file_id, local_folder=local_path,
                                       file_filter=pan_file_filter)


def walk_file_list(ali_api: Aligo, handle: Callable[[str, BaseFile], None], parent_file_id: str = 'root'):
    """
    遍历文件
    :param ali_api: Aligo
    :param handle: 文件处理函数，如：def handle(path: str, f: BaseFile):
    :param parent_file_id: 文件路径
    :return:
    """
    ali_api.walk_files(handle, parent_file_id=parent_file_id)



from starlette.responses import Response
from starlette.templating import Jinja2Templates, _TemplateResponse
from starlette.requests import Request

from afeng_tools.application_tool import settings_tools
from afeng_tools.application_tool.settings_enum import SettingsKeyEnum

jinja2_templates = Jinja2Templates(directory=settings_tools.get_config(SettingsKeyEnum.server_template_path))


def create_template_response(request: Request, template_file: str, context: dict = None) -> _TemplateResponse:
    """
    创建模板响应
    :param request: Request
    :param template_file: 模板文件
    :param context: 上下文内容
    :return:
    """
    if isinstance(context, Response):
        return context
    if not context:
        context = dict()
    if 'request' not in context:
        context['request'] = request
    return jinja2_templates.TemplateResponse(template_file, context=context)



import enum
import os
from typing import Type, TypeVar, Callable

from fastapi import APIRouter, Body, Depends, Form
from starlette.requests import Request

from afeng_tools.fastapi_tool.admin.admin_router_model import AdminRouterModel
from afeng_tools.fastapi_tool.admin.admin_views_template import AdminViewsTemplate
from afeng_tools.jinja2_tool import jinja2_tools
from afeng_tools.pydantic_tool.pydantic_tools import create_model_by_po
from afeng_tools.sqlalchemy_tool.core.sqlalchemy_base_model import Model


def _create_json_proxy_func(source_func: Callable, orm_model_class: Type[Model], add_flag: bool = False):
    """
    创建代理函数
    :param source_func: 需要被代理的函数
    :param orm_model_class: orm模型类
    :param add_flag: 是否是添加数据
    :return: 代理函数
    """
    ModelForm = TypeVar('ModelForm', bound=create_model_by_po(orm_model_class, add_flag=add_flag))

    async def proxy_func(request: Request, form_data: ModelForm = Body()):
        return await source_func(form_data, request=request)

    return proxy_func


def _create_form_proxy_old_func(source_func: Callable, orm_model_class: Type[Model], add_flag: bool = False):
    """
    创建代理函数
    :param source_func: 需要被代理的函数
    :param orm_model_class: orm模型类
    :param add_flag: 是否是添加数据
    :return: 代理函数
    """
    model_class = create_model_by_po(orm_model_class, add_flag=add_flag)
    ModelForm = TypeVar('ModelForm', bound=model_class)

    async def convert_form_to_model(request: Request) -> ModelForm:
        form_data = await request.form()
        return model_class(**{tmp_key: tmp_value for tmp_key, tmp_value in form_data.items()})

    async def proxy_func(request: Request, form_data: ModelForm = Depends(convert_form_to_model)):
        return await source_func(form_data, request=request)

    return proxy_func


def _create_param_convert_depend_func(module_code: str, orm_model_class: Type[Model]) -> Callable:
    params_dict = {c.name: c for c in orm_model_class.__table__.columns}
    template_file = os.path.join(os.path.dirname(__file__), 'auto_covert_func.template')
    enum_import = []
    for tmp_value in params_dict.values():
        if issubclass(tmp_value.type.python_type, enum.Enum):
            enum_import.append(
                f"from {tmp_value.type.python_type.__module__} import {tmp_value.type.python_type.__name__}")
    params_list = []
    for tmp_field, tmp_column in params_dict.items():
        tmp_type = tmp_column.type.python_type
        if issubclass(tmp_type, list):
            tmp_item_type = tmp_column.type.item_type.python_type
            params_list.append(
                f"{tmp_field}: Optional[{tmp_column.type.python_type.__name__}[{tmp_item_type.__name__}]] = Form(title='{tmp_column.comment}', default=None)")
        else:
            if issubclass(tmp_type, enum.Enum):
                for tmp_enum in tmp_type:
                    tmp_type._value2member_map_[tmp_enum.name] = tmp_enum
            params_list.append(
                f"{tmp_field}: Optional[{tmp_column.type.python_type.__name__}] = Form(title='{tmp_column.comment}', default=None)")

    args_list = [f"'{tmp}': {tmp}" for tmp in params_dict.keys()]
    template_content = jinja2_tools.format_template(template_file, context={
        'module_code': module_code,
        'enum_import_str': '\n'.join(enum_import),
        'params_str': ', '.join(params_list),
        'args_str': '{' + (', '.join(args_list)) + '}'
    })
    exec(template_content)
    return eval(f'convert_form_to_{module_code}_model_func')


def _create_form_proxy_func(module_code: str, source_func: Callable, orm_model_class: Type[Model],
                            add_flag: bool = False):
    """
    创建代理函数
    :param source_func: 需要被代理的函数
    :param orm_model_class: orm模型类
    :param add_flag: 是否是添加数据
    :return: 代理函数
    """
    model_class = create_model_by_po(orm_model_class, add_flag=add_flag)
    param_convert_depend_func = _create_param_convert_depend_func(module_code=module_code,
                                                                  orm_model_class=orm_model_class)

    async def proxy_func(request: Request,
                         form_data: dict = Depends(param_convert_depend_func)):
        old_form_data = await request.form()
        if old_form_data:
            array_field_dict = dict()
            for tmp_key in old_form_data.keys():
                if tmp_key.endswith('[]'):
                    tmp_new_key = tmp_key.removesuffix('[]')
                    if not array_field_dict.get(tmp_new_key):
                        array_field_dict[tmp_new_key] = old_form_data.getlist(tmp_key)
            form_data.update(array_field_dict)
        return await source_func(request, model_class(**form_data))

    return proxy_func


def create_admin_router(admin_router_model: AdminRouterModel) -> tuple[APIRouter, APIRouter]:
    """
    添加管理路由
    :param admin_router_model:
    :return: 管理页面路由，管理api路由
    """
    module_code: str = admin_router_model.module_code
    orm_model_class: Type[Model] = admin_router_model.orm_model_class
    admin_views_template: AdminViewsTemplate = admin_router_model.admin_views_template
    method_dict = dict()
    if len(type(admin_views_template).__mro__) > 2 and type(admin_views_template).__mro__[1] == AdminViewsTemplate:
        method_dict = [tmp for tmp in type(admin_views_template).__dict__.keys()]
        method_dict.remove('__module__')
        method_dict.remove('__doc__')
    page_router = APIRouter(prefix=f'/admin/{module_code}', tags=[f'管理-{tmp}' for tmp in admin_router_model.tag_list])
    page_router.get('')(admin_views_template.index_page)
    page_router.get('/index')(admin_views_template.index_page)
    page_router.get('/home')(admin_views_template.index_page)
    page_router.get('/add')(admin_views_template.add_page)
    page_router.get('/update/{id_value}')(admin_views_template.update_page)
    page_router.get('/show')(admin_views_template.show_page)

    api_router = APIRouter(prefix=f'/admin/api/{module_code}',
                           tags=[f'管理Api-{tmp}' for tmp in admin_router_model.tag_list])
    if 'add_data' in method_dict:
        api_router.post('/add')(admin_views_template.add_data)
    else:
        api_router.post('/add')(
            _create_form_proxy_func(module_code, admin_views_template.add_data, orm_model_class, add_flag=True))
    if 'update_data' in method_dict:
        api_router.post('/update')(admin_views_template.update_data)
    else:
        api_router.post('/update')(
            _create_form_proxy_func(module_code, admin_views_template.update_data, orm_model_class))
    api_router.get('/get/{id_value}')(admin_views_template.get_data)
    api_router.post('/delete/{id_value}')(admin_views_template.delete_data)
    api_router.post('/more_delete')(admin_views_template.more_delete_data)
    api_router.get('/list')(admin_views_template.list_data)
    return page_router, api_router



"""
东方财富综合评价模块
"""
from afeng_tools.east_money_tool.tool.east_money_http_tools import get_datacenter_data
from afeng_tools.east_money_tool.tool.east_money_load_auth_tools import load_stock_comment_auth_file


def load_zf_pm(stock_code):
    cookie_file, header_file = load_stock_comment_auth_file(stock_code)
    json_data = get_datacenter_data(stock_code='001268',
                                    opt_dict={'reportName': 'RPT_CUSTOM_STOCK_PK'},
                                    cookie_file=cookie_file,
                                    header_file=header_file)
    print(json_data)
def load_zdyc():
    """涨跌预测"""
    pass

def load_lspf():
    pass

def load_zhpm():
    pass


"""
百度盘工具
"""
import json
import os.path
import re
from datetime import datetime

from afeng_tools.baidu_pan_tool import baidu_pan_settings
from afeng_tools.baidu_pan_tool.baidu_pan_enum import BaidupanConfigKeyEnum
from afeng_tools.baidu_pan_tool.core.baidu_pan_models import TokenInfo, MultimediaFileInfo, UploadCreateResult
from afeng_tools.baidu_pan_tool.tools import file_path_tools, file_upload_tools
from afeng_tools.encryption_tool import hashlib_tools
from afeng_tools.file_tool import file_tools, tmp_file_tools
from afeng_tools.http_tool import http_download_tools
from afeng_tools.log_tool.loguru_tools import get_logger, log_error

logger = get_logger()

try:
    from afeng_tools.baidu_pan_tool.tools import auth_tools
except ModuleNotFoundError:
    from afeng_tools.baidu_pan_tool.tools import auth_web_tools as auth_tools


def _save_token(token_info: TokenInfo):
    """保存token到本地"""
    file_tools.save_file(json.dumps(token_info.model_dump(mode='json')).encode('utf-8'),
                         baidu_pan_settings.get_config(BaidupanConfigKeyEnum.token_file), binary_flag=True)


def _is_need_refresh(token_info: TokenInfo):
    """
    判断token是否需要刷新
    :param token_info:
    :return: 是否需要刷新，默认是False
    """
    space_time = datetime.now() - token_info.token_time
    space_seconds = space_time.total_seconds()
    if space_seconds - token_info.expires_in > 3600:
        return True
    return False


def get_access_token() -> str:
    """获取Access Token"""
    if os.path.exists(baidu_pan_settings.get_config(BaidupanConfigKeyEnum.token_file)):
        # token文件转换为token
        token_info_bytes = file_tools.read_file(baidu_pan_settings.get_config(BaidupanConfigKeyEnum.token_file),
                                                binary_flag=True)
        token_info = TokenInfo.model_validate_json(token_info_bytes)
        # 判断已有的token是否需要刷新
        if _is_need_refresh(token_info):
            # 1、刷新token
            token_info = auth_tools.refresh_token(token_info.refresh_token,
                                                  baidu_pan_settings.get_config(BaidupanConfigKeyEnum.app_key),
                                                  baidu_pan_settings.get_config(BaidupanConfigKeyEnum.secret_key))
            # 2、保存刷新后的token
            _save_token(token_info)
        return token_info.access_token
    else:
        # 获取授权码Code
        auth_code = auth_tools.get_authorization_code(
            app_id=baidu_pan_settings.get_config(BaidupanConfigKeyEnum.app_id),
            app_key=baidu_pan_settings.get_config(BaidupanConfigKeyEnum.app_key))
        # 换取AccessToken凭证
        token_info = auth_tools.get_token_authorization_code(auth_code=auth_code,
                                                             app_key=baidu_pan_settings.get_config(
                                                                 BaidupanConfigKeyEnum.app_key),
                                                             secret_key=baidu_pan_settings.get_config(
                                                                 BaidupanConfigKeyEnum.secret_key))
        # 保存token到本地
        _save_token(token_info)
        return token_info.access_token


def list_file(access_token: str, dir_path: str = '/', order: str = 'name', desc: int = 0,
              start: int = 0, limit: int = 100, web: int = 0, folder: int = 0,
              show_empty: int = 1) -> MultimediaFileInfo:
    """
    获取文件列表: 本接口用于获取用户网盘中指定目录下的文件列表。返回的文件列表支持排序、分页等操作。
    :param access_token: Access Token
    :param dir_path: 需要list的目录，以/开头的绝对路径, 默认为/
    :param order: 排序字段：默认为name；
                time表示先按文件类型排序，后按修改时间排序；
                name表示先按文件类型排序，后按文件名称排序；(注意，此处排序是按字符串排序的)
                size表示先按文件类型排序，后按文件大小排序。
    :param desc: 默认为升序，设置为1实现降序, 排序的对象是当前目录下所有文件，不是当前分页下的文件
    :param start: 起始位置，从0开始
    :param limit: 查询数目，默认为1000，建议最大不超过1000
    :param web: 值为1时，返回dir_empty属性和缩略图数据
    :param folder: 是否只返回文件夹，0 返回所有，1 只返回文件夹，且属性只返回path字段
    :param show_empty: 是否返回dir_empty属性，0 不返回，1 返回
    :return:
    """
    return file_path_tools.list_all_by_page(access_token=access_token, dir_path=dir_path,
                                            order=order, desc=desc, start=start, limit=limit,
                                            web=web, folder=folder, show_empty=show_empty)


def get_share_url(share_msg: str) -> str | None:
    """
    提取分享信息中的分享链接
    :param share_msg: 分享信息，如： 链接:https://pan.baidu.com/s/1NnNRMgCdzQbBFXtx_xGezg 提取码:2112
    :return: 分享链接，如：https://pan.baidu.com/s/1NnNRMgCdzQbBFXtx_xGezg?pwd=2112
    """
    url_re_search = re.search('(https://pan\.baidu\.com/s/[\w-]*)', share_msg)
    if url_re_search is None:
        url_re_search = re.search('(https://pan\.baidu\.com/share/[/?=&\w-]*)', share_msg)
    if url_re_search is None:
        return
    share_url = url_re_search.group(1)
    if 'pwd=' in share_url:
        return share_url
    pwd_re_search = re.search('提取码:(\w{4})', share_msg)
    if pwd_re_search:
        share_pwd = pwd_re_search.group(1)
        return share_url + '?pwd=' + share_pwd
    return share_url


def save_to_pan(file_url: str, pan_path: str, include_filename: bool = False) -> UploadCreateResult:
    """
    保存到网盘
    :param file_url: 文件的url
    :param pan_path: 网盘存储的目录
    :param include_filename: 路径是否包含文件名
    :return: 存储结果
    """
    access_token = get_access_token()
    tmp_path = tmp_file_tools.get_user_tmp_dir()
    tmp_file_name = file_url.rsplit('/', maxsplit=1)[-1].split('?', maxsplit=1)[0]
    tmp_file = os.path.join(tmp_path, tmp_file_name)
    try:
        http_download_tools.download_chunk_file(download_url=file_url,
                                                save_path=tmp_path,
                                                save_file_name=tmp_file_name)
        if not include_filename:
            tmp_pan_file_name = hashlib_tools.calc_file_md5(tmp_file) + '.' + tmp_file_name.rsplit('.')[1]
            pan_path = f'{pan_path}/{tmp_pan_file_name}'
        return file_upload_tools.upload_file(access_token, local_file=tmp_file, pan_path=pan_path, rtype=3)
    except Exception as e:
        log_error(logger, f'[百度网盘]下载文件[{file_url}], 保存文件到百度网盘[{pan_path}]出现异常', e)
    finally:
        if os.path.exists(tmp_file):
            os.remove(tmp_file)


def upload_to_pan(local_file: str, pan_path: str, include_filename: bool = False) -> UploadCreateResult:
    """
    保存到网盘
    :param local_file: 本地文件
    :param pan_path: 网盘存储的目录
    :param include_filename: 路径是否包含文件名
    :return: 存储结果
    """
    error_chars = '?|"><:*'
    for tmp in error_chars:
        pan_path = pan_path.replace(tmp, '_')
    access_token = get_access_token()
    tmp_file_name = local_file.rsplit('/', maxsplit=1)[-1].split('?', maxsplit=1)[0]
    try:
        if not include_filename:
            tmp_pan_file_name = hashlib_tools.calc_md5(local_file) + '.' + tmp_file_name.rsplit('.')[1]
            pan_path = f'{pan_path}/{tmp_pan_file_name}'
        return file_upload_tools.upload_file(access_token, local_file=local_file, pan_path=pan_path, rtype=3)
    except Exception as e:
        log_error(logger, f'[百度网盘]文件[{local_file}], 保存文件到百度网盘[{pan_path}]出现异常', e)



"""
微信数据解密: https://developers.weixin.qq.com/miniprogram/dev/framework/open-ability/signature.html#%E5%8A%A0%E5%AF%86%E6%95%B0%E6%8D%AE%E8%A7%A3%E5%AF%86%E7%AE%97%E6%B3%95
安装：
- windows平台： pip install pycryptodome
- linux平台： pip install pycrypto
    - 如果出现：ModuleNotFoundError: No module named 'Crypto'，则执行以下操作：
        pip install pycryptodome
"""
import base64
import json
from Crypto.Cipher import AES


class WXBizDataCrypt:
    def __init__(self, app_id, session_key):
        self.app_id = app_id
        self.session_key = session_key

    def decrypt(self, encrypted_data, iv: str):
        # base64 decode
        sessionKey = base64.b64decode(self.session_key)
        encryptedData = base64.b64decode(encrypted_data)
        iv = base64.b64decode(iv)
        cipher = AES.new(sessionKey, AES.MODE_CBC, iv)
        _unpad = lambda s: s[:-ord(s[len(s) - 1:])]
        decrypted = json.loads(_unpad(cipher.decrypt(encryptedData)))
        if decrypted['watermark']['appid'] != self.app_id:
            raise Exception('Invalid Buffer')
        return decrypted




from afeng_tools.fastapi_tool import fastapi_router_tools
from afeng_tools.web_tool import response_tools
from afeng_tools.weixin_min_program.core import weixin_mp_service
from afeng_tools.weixin_min_program.core.weixin_mp_models import WxUserProfile

router = fastapi_router_tools.create_router(prefix='/api', tags=['微信接口'])


@router.get("/wx/mp/login")
async def weixin_login(app_id: str, js_code: str):
    """
    小程序登录
    :param app_id: 小程序 App id
    :param js_code: 通过js中的wx.login()获取到的code值
    :return: token
    """
    return response_tools.create_json_response(weixin_mp_service.login_code2Session(app_id, js_code))


@router.post("/wx/mp/userinfo")
async def weixin_login(user_profile: WxUserProfile):
    """
    小程序登录
    :param user_profile: 授权登录后获取到的用户信息
    :return: token
    """
    return response_tools.create_json_response(weixin_mp_service.decrypt_userinfo(user_profile.app_id, user_profile.token, user_profile))



"""
HTTP工具
- 安装：pip install lxml -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
"""

from typing import Optional

import requests
from lxml import etree

from afeng_tools.http_tool.decorator.http_auto_retry_tools import auto_retry
from afeng_tools.log_tool.loguru_tools import get_logger

logger = get_logger()


@auto_retry(max_retries=3)
def get_html_text(url, params: Optional[dict] = None, headers: Optional[dict] = None, retry: int = 3, timeout: int = 3000):
    """获取请求得到的响应文本内容"""
    if headers is None:
        headers = dict()
    if 'User-Agent' not in headers:
        headers[
            'User-Agent'] = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/107.0.0.0 Safari/537.36 Edg/107.0.1418.24'
    logger.info(f'下载html[{url}][{params}]')
    response = requests.get(url, params, timeout=timeout, headers=headers)
    if response.status_code == 200:
        response.encoding = 'utf-8'
        return response.text
    elif retry > 0:
        return get_html_text(url, params=params, headers=headers, retry=retry - 1, timeout=timeout)


def get_html_tree(url):
    """获取xpath的html_tree, 获取到后可以：html_tree.xpath('//*[@class="test"]')"""
    content = get_html_text(url)
    html_tree = etree.HTML(content)
    return html_tree


def get_m3u8_content(m3u8_url: str, headers: dict[str, str]) -> bytes:
    """获取m3u8内容"""
    response = requests.get(m3u8_url, headers=headers)
    if response.status_code == 200:
        return response.content



"""
TTS语音工具： pip install pyttsx3 -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
"""
import pyttsx3


def text_to_speech(text):
    engine = pyttsx3.init()
    engine.say(text)
    engine.runAndWait()


if __name__ == '__main__':
    text_to_speech('你好，我是中国人')



"""
http自动重试工具
"""
import time
from typing import Callable

from afeng_tools.log_tool.loguru_tools import get_logger, log_error

logger = get_logger()


def auto_retry(max_retries: int, retry_delay: float = 0.5):
    """
    自动重试装饰器
    :param max_retries: 最大重试次数
    :param retry_delay: 延迟多少秒重试
    :return: func_wrap
    """
    def func_wrap(func: Callable):
        def func_inner(*args, **kwargs):
            try:
                if isinstance(func, Callable):
                    result = func(*args, **kwargs)
                else:
                    result = func.__func__(*args, **kwargs)
                return result
            except Exception as e:
                log_error(logger, f'[http-retry-{max_retries}][{func.__name__}]-[{args}]-[{kwargs}]', e)
                if max_retries > 0:
                    time.sleep(retry_delay)
                    return auto_retry(max_retries=max_retries - 1, retry_delay=retry_delay)(func)(*args, **kwargs)
                else:
                    raise e
        return func_inner
    return func_wrap



from datetime import datetime

from pydantic import BaseModel


class WeixinTokenInfo(BaseModel):
    """Token信息"""
    # 获取到的凭证
    access_token: str
    # 凭证有效时间，单位：秒
    expires_in: int
    # token获取的时间，用于判断token是否过期
    token_time: datetime


from sqlalchemy import Column, Integer, String

from afeng_tools.sqlalchemy_tool.core.sqlalchemy_base_model import Model


class WeixinMsgBaseModel(Model):
    """微信消息基本信息表"""
    __abstract__ = True
    openid = Column(String(125), comment='OpenID', nullable=False)
    to_user = Column(String(250), comment='开发者微信号', nullable=False)
    from_user = Column(String(250), comment='发送方帐号（一个OpenID）', nullable=False)
    create_time = Column(Integer, comment='消息创建时间 （整型）', nullable=False)
    # 消息类型，文本为text, 图片为image，语音为voice，视频为video,小视频为shortvideo，地理位置为location，链接为link
    msg_type = Column(String(30), comment='消息类型', nullable=False)
    msg_id = Column(String(64), comment='消息id，64位整型', nullable=False)
    msg_data_id = Column(String(64), comment='消息的数据ID（消息如果来自文章时才有）', nullable=True)
    idx = Column(String(64), comment='多图文时第几篇文章，从1开始（消息如果来自文章时才有）', nullable=True)


class WeixinTextMsg(WeixinMsgBaseModel):
    """文本消息表"""
    __tablename__ = 'weixin_text_msg'
    msg_content = Column(String(850), comment='文本消息内容', nullable=False)

    def __repr__(self):
        return self.msg_content[:50]


class WeixinImageMsg(WeixinMsgBaseModel):
    """图片消息表"""
    __tablename__ = 'weixin_image_msg'
    pic_url = Column(String(500), comment='图片链接（由系统生成）', nullable=False)
    media_id = Column(String(250), comment='图片消息媒体id，可以调用获取临时素材接口拉取数据。', nullable=False)

    def __repr__(self):
        return self.pic_url


class WeixinVoiceMsg(WeixinMsgBaseModel):
    """语音消息表"""
    __tablename__ = 'weixin_voice_msg'
    media_id = Column(String(250), comment='语音消息媒体id，可以调用获取临时素材接口拉取数据。', nullable=False)
    voice_format = Column(String(30), comment='语音格式，如amr，speex等', nullable=False)
    recognition = Column(String(500), comment='语音识别结果，UTF8编码', nullable=True)

    def __repr__(self):
        return self.media_id


class WeixinVideoMsg(WeixinMsgBaseModel):
    """视频消息表"""
    __tablename__ = 'weixin_video_msg'
    media_id = Column(String(250), comment='视频消息媒体id，可以调用获取临时素材接口拉取数据。', nullable=False)
    thumb_media_id = Column(String(255), comment='视频消息缩略图的媒体id，可以调用多媒体文件下载接口拉取数据。',
                            nullable=False)

    def __repr__(self):
        return self.media_id


class WeixinLocationMsg(WeixinMsgBaseModel):
    """地理位置消息表"""
    __tablename__ = 'weixin_location_msg'
    location_x = Column(String(20), comment='地理位置纬度', nullable=False)
    location_y = Column(String(20), comment='地理位置经度', nullable=False)
    scale = Column(String(10), comment='地图缩放大小', nullable=False)
    label = Column(String(150), comment='地理位置信息', nullable=False)

    def __repr__(self):
        return self.label


class WeixinLinkMsg(WeixinMsgBaseModel):
    """链接消息表"""
    __tablename__ = 'weixin_link_msg'
    title = Column(String(100), comment='消息标题', nullable=False)
    description = Column(String(500), comment='消息描述', nullable=False)
    url = Column(String(500), comment='消息链接', nullable=False)

    def __repr__(self):
        return self.title



# 创建Flask应用
from util.flask import flask_app_tools
from web.urls import url_pattern_list

app = flask_app_tools.create_app(url_pattern_list)


def run_app():
    """运行app"""
    flask_app_tools.run_app(app)


if __name__ == '__main__':
    run_app()
